@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,18 +20,18 @@ import numpy as np
 import pandas as pd
 
 from .. import opcodes as OperandDef
-from ..operands import OperandStage, OutputType
-from ..serialize import ValueType, AnyField, BoolField, Int32Field, KeyField, ListField
-from ..utils import get_shuffle_input_keys_idxes
+from ..core import OutputType
+from ..core.operand import OperandStage, MapReduceOperand
+from ..serialization.serializables import FieldTypes, AnyField, \
+    BoolField, Int32Field, KeyField, ListField
 from .core import SERIES_CHUNK_TYPE
 from .utils import hash_dtypes, filter_dtypes
-from .operands import DataFrameMapReduceOperand, DataFrameOperandMixin, \
-    DataFrameShuffleProxy
+from .operands import DataFrameOperandMixin, DataFrameShuffleProxy
 from .utils import parse_index, split_monotonic_index_min_max, \
     build_split_idx_to_origin_idx, filter_index_value, hash_index
 
 
-class DataFrameIndexAlign(DataFrameMapReduceOperand, DataFrameOperandMixin):
+class DataFrameIndexAlign(MapReduceOperand, DataFrameOperandMixin):
     _op_type_ = OperandDef.DATAFRAME_INDEX_ALIGN
 
     _index_min = AnyField('index_min')
@@ -44,14 +44,13 @@ class DataFrameIndexAlign(DataFrameMapReduceOperand, DataFrameOperandMixin):
     _column_max = AnyField('column_max')
     _column_max_close = BoolField('column_max_close')
     _column_shuffle_size = Int32Field('column_shuffle_size')
-    _column_shuffle_segments = ListField('column_shuffle_segments', ValueType.series)
+    _column_shuffle_segments = ListField('column_shuffle_segments', FieldTypes.series)
 
     _input = KeyField('input')
 
     def __init__(self, index_min_max=None, index_shuffle_size=None, column_min_max=None,
                  column_shuffle_size=None, column_shuffle_segments=None,
-                 sparse=None, dtype=None, dtypes=None, gpu=None, stage=None, shuffle_key=None,
-                 output_types=None, **kw):
+                 sparse=None, dtype=None, dtypes=None, output_types=None, **kw):
         if index_min_max is not None:
             kw.update(dict(_index_min=index_min_max[0], _index_min_close=index_min_max[1],
                            _index_max=index_min_max[2], _index_max_close=index_min_max[3]))
@@ -61,8 +60,7 @@ class DataFrameIndexAlign(DataFrameMapReduceOperand, DataFrameOperandMixin):
         super().__init__(
             _index_shuffle_size=index_shuffle_size, _column_shuffle_size=column_shuffle_size,
             _column_shuffle_segments=column_shuffle_segments, _sparse=sparse,
-            _dtype=dtype, _dtypes=dtypes, _gpu=gpu, _stage=stage, _shuffle_key=shuffle_key,
-            _output_types=output_types, **kw)
+            _dtype=dtype, _dtypes=dtypes, _output_types=output_types, **kw)
 
     @property
     def index_min(self):
@@ -126,8 +124,7 @@ class DataFrameIndexAlign(DataFrameMapReduceOperand, DataFrameOperandMixin):
         super()._set_inputs(inputs)
         self._input = self._inputs[0]
 
-    def _build_map_chunk_kw(self, **kw):
-        inputs = self.inputs
+    def build_map_chunk_kw(self, inputs, **kw):
         if kw.get('index_value', None) is None and inputs[0].index_value is not None:
             input_index_value = inputs[0].index_value
             index_min_max = self.index_min_max
@@ -136,28 +133,32 @@ class DataFrameIndexAlign(DataFrameMapReduceOperand, DataFrameOperandMixin):
             else:
                 kw['index_value'] = parse_index(inputs[0].index_value.to_pandas(),
                                                 input_index_value, type(self).__name__)
-        if kw.get('columns_value', None) is None and getattr(inputs[0], 'columns_value', None) is not None:
-            input_columns_value = inputs[0].columns_value
-            input_dtypes = inputs[0].dtypes
-            column_min_max = self.column_min_max
-            if column_min_max is not None:
-                kw['columns_value'] = filter_index_value(input_columns_value, column_min_max,
-                                                         store_data=True)
-            else:
-                kw['columns_value'] = parse_index(inputs[0].columns_value.to_pandas(), input_columns_value,
-                                                  type(self).__name__)
-            kw['dtypes'] = input_dtypes[kw['columns_value'].to_pandas()]
-            column_shuffle_size = self.column_shuffle_size
-            if column_shuffle_size is not None:
-                self._column_shuffle_segments = hash_dtypes(input_dtypes, column_shuffle_size)
-        if kw.get('dtype', None) and getattr(inputs[0], 'dtype', None) is not None:
-            kw['dtype'] = inputs[0].dtype
-        if kw.get('name', None) and getattr(inputs[0], 'name', None) is not None:
-            kw['name'] = inputs[0].dtype
+        if self.output_types[0] == OutputType.dataframe:
+            if kw.get('columns_value', None) is None and \
+                    getattr(inputs[0], 'columns_value', None) is not None:
+                input_columns_value = inputs[0].columns_value
+                input_dtypes = inputs[0].dtypes
+                column_min_max = self.column_min_max
+                if column_min_max is not None:
+                    kw['columns_value'] = filter_index_value(
+                        input_columns_value, column_min_max, store_data=True)
+                else:
+                    kw['columns_value'] = parse_index(inputs[0].columns_value.to_pandas(),
+                                                      input_columns_value,
+                                                      type(self).__name__)
+                kw['dtypes'] = input_dtypes[kw['columns_value'].to_pandas()]
+                column_shuffle_size = self.column_shuffle_size
+                if column_shuffle_size is not None:
+                    self._column_shuffle_segments = hash_dtypes(input_dtypes, column_shuffle_size)
+        else:
+            if kw.get('dtype', None) is None and getattr(inputs[0], 'dtype', None) is not None:
+                kw['dtype'] = inputs[0].dtype
+            if kw.get('name', None) is None and getattr(inputs[0], 'name', None) is not None:
+                kw['name'] = inputs[0].name
         return kw
 
-    def _build_reduce_chunk_kw(self, index, **kw):
-        inputs = self.inputs
+    def build_reduce_chunk_kw(self, inputs, index, **kw):
+        kw['index'] = index
         if kw.get('index_value', None) is None and inputs[0].inputs[0].index_value is not None:
             index_align_map_chunks = inputs[0].inputs
             if index_align_map_chunks[0].op.index_min_max is not None:
@@ -166,34 +167,33 @@ class DataFrameIndexAlign(DataFrameMapReduceOperand, DataFrameOperandMixin):
                                                        index_align_map_chunks[0].op.index_min_max)
             else:
                 # shuffle on index
-                kw['index_value'] = parse_index(index_align_map_chunks[0].index_value.to_pandas(),
-                                                [c.key for c in index_align_map_chunks], type(self).__name__)
-        if kw.get('columns_value', None) is None and getattr(inputs[0].inputs[0], 'columns_value', None) is not None:
-            index_align_map_chunks = inputs[0].inputs
-            if index_align_map_chunks[0].op.column_min_max is not None:
-                # shuffle on index
-                kw['columns_value'] = filter_index_value(index_align_map_chunks[0].columns_value,
-                                                         index_align_map_chunks[0].op.column_min_max,
-                                                         store_data=True)
-                kw['dtypes'] = index_align_map_chunks[0].dtypes[kw['columns_value'].to_pandas()]
-            else:
-                # shuffle on columns
-                all_dtypes = [c.op.column_shuffle_segments[index[1]] for c in index_align_map_chunks
-                              if c.index[0] == index_align_map_chunks[0].index[0]]
-                kw['dtypes'] = pd.concat(all_dtypes)
-                kw['columns_value'] = parse_index(kw['dtypes'].index, store_data=True)
-        if kw.get('dtype', None) and getattr(inputs[0].inputs[0], 'dtype', None) is not None:
-            kw['dtype'] = inputs[0].inputs[0].dtype
-        if kw.get('name', None) and getattr(inputs[0].inputs[0], 'name', None) is not None:
-            kw['name'] = inputs[0].inputs[0].dtype
-        return kw
-
-    def _create_chunk(self, output_idx, index, **kw):
-        if self.stage == OperandStage.map:
-            kw = self._build_map_chunk_kw(**kw)
+                kw['index_value'] = parse_index(
+                    index_align_map_chunks[0].index_value.to_pandas(),
+                    [c.key for c in index_align_map_chunks], type(self).__name__)
+        if self.output_types[0] == OutputType.dataframe:
+            if kw.get('columns_value', None) is None and \
+                    getattr(inputs[0].inputs[0], 'columns_value', None) is not None:
+                index_align_map_chunks = inputs[0].inputs
+                if index_align_map_chunks[0].op.column_min_max is not None:
+                    # shuffle on index
+                    kw['columns_value'] = filter_index_value(
+                        index_align_map_chunks[0].columns_value,
+                        index_align_map_chunks[0].op.column_min_max,
+                        store_data=True)
+                    kw['dtypes'] = index_align_map_chunks[0].dtypes[kw['columns_value'].to_pandas()]
+                else:
+                    # shuffle on columns
+                    all_dtypes = [c.op.column_shuffle_segments[index[1]]
+                                  for c in index_align_map_chunks
+                                  if c.index[0] == index_align_map_chunks[0].index[0]]
+                    kw['dtypes'] = pd.concat(all_dtypes)
+                    kw['columns_value'] = parse_index(kw['dtypes'].index, store_data=True)
         else:
-            kw = self._build_reduce_chunk_kw(index, **kw)
-        return super()._create_chunk(output_idx, index, **kw)
+            if kw.get('dtype', None) is None and getattr(inputs[0].inputs[0], 'dtype', None) is not None:
+                kw['dtype'] = inputs[0].inputs[0].dtype
+            if kw.get('name', None) is None and getattr(inputs[0].inputs[0], 'name', None) is not None:
+                kw['name'] = inputs[0].inputs[0].name
+        return kw
 
     @classmethod
     def execute_map(cls, ctx, op):
@@ -224,8 +224,7 @@ class DataFrameIndexAlign(DataFrameMapReduceOperand, DataFrameOperandMixin):
                 ctx[chunk.key] = df.loc[filters[0][0]]
             else:
                 for index_idx, index_filter in enumerate(filters[0]):
-                    group_key = str(index_idx)
-                    ctx[(chunk.key, group_key)] = df.loc[index_filter]
+                    ctx[chunk.key, (index_idx,)] = df.loc[index_filter]
             return
 
         if op.column_shuffle_size == -1:
@@ -249,13 +248,13 @@ class DataFrameIndexAlign(DataFrameMapReduceOperand, DataFrameOperandMixin):
         elif len(filters[0]) == 1:
             # shuffle on columns
             for column_idx, column_filter in enumerate(filters[1]):
-                group_key = ','.join([str(chunk.index[0]), str(column_idx)])
-                ctx[(chunk.key, group_key)] = df.loc[filters[0][0], column_filter]
+                shuffle_index = (chunk.index[0], column_idx)
+                ctx[chunk.key, shuffle_index] = df.loc[filters[0][0], column_filter]
         elif len(filters[1]) == 1:
             # shuffle on index
             for index_idx, index_filter in enumerate(filters[0]):
-                group_key = ','.join([str(index_idx), str(chunk.index[1])])
-                ctx[(chunk.key, group_key)] = df.loc[index_filter, filters[1][0]]
+                shuffle_index = (index_idx, chunk.index[1])
+                ctx[chunk.key, shuffle_index] = df.loc[index_filter, filters[1][0]]
         else:
             # full shuffle
             shuffle_index_size = op.index_shuffle_size
@@ -264,15 +263,12 @@ class DataFrameIndexAlign(DataFrameMapReduceOperand, DataFrameOperandMixin):
             out_index_columns = itertools.product(*filters)
             for out_idx, out_index_column in zip(out_idxes, out_index_columns):
                 index_filter, column_filter = out_index_column
-                group_key = ','.join(str(i) for i in out_idx)
-                ctx[(chunk.key, group_key)] = df.loc[index_filter, column_filter]
+                ctx[chunk.key, out_idx] = df.loc[index_filter, column_filter]
 
     @classmethod
-    def execute_reduce(cls, ctx, op):
+    def execute_reduce(cls, ctx, op: "DataFrameIndexAlign"):
         chunk = op.outputs[0]
-        input_keys, input_idxes = get_shuffle_input_keys_idxes(op.inputs[0])
-        input_idx_to_df = {idx: ctx[inp_key, ','.join(str(ix) for ix in chunk.index)]
-                           for inp_key, idx in zip(input_keys, input_idxes)}
+        input_idx_to_df = dict(op.iter_mapper_data_with_index(ctx))
         row_idxes = sorted({idx[0] for idx in input_idx_to_df})
         if chunk.ndim == 2:
             col_idxes = sorted({idx[1] for idx in input_idx_to_df})
@@ -420,8 +416,9 @@ def _get_chunk_index_min_max(index_chunks):
         max_val = chunk.max_val
         max_val_close = chunk.max_val_close
         if min_val is None or max_val is None:
-            return
-        chunk_index_min_max.append((min_val, min_val_close, max_val, max_val_close))
+            chunk_index_min_max.append((None, True, None, True))
+        else:
+            chunk_index_min_max.append((min_val, min_val_close, max_val, max_val_close))
     return chunk_index_min_max
 
 
@@ -442,8 +439,6 @@ def _get_monotonic_chunk_index_min_max(index, index_chunks):
 
 def _need_align_map(input_chunk, index_min_max, column_min_max,
                     dummy_index_splits=False, dummy_column_splits=False):
-    if not dummy_index_splits:
-        assert not index_min_max[0] is None and not index_min_max[2] is None
     if isinstance(input_chunk, SERIES_CHUNK_TYPE):
         if input_chunk.index_value is None:
             return True
@@ -502,8 +497,7 @@ def _calc_axis_splits(left_axis, right_axis, left_axis_chunks, right_axis_chunks
             left_splits, right_splits = split_monotonic_index_min_max(
                 left_chunk_index_min_max, left_increase, right_chunk_index_min_max, right_increase)
         splits = _AxisMinMaxSplitInfo(left_splits, left_increase, right_splits, right_increase)
-        nsplits = [np.nan for _ in itertools.chain(*left_splits)]
-        return splits, nsplits
+        return splits, None
 
 
 def _build_dummy_axis_split(chunk_shape):
@@ -529,7 +523,8 @@ def _gen_series_chunks(splits, out_shape, left_or_right, series):
                 align_op = DataFrameIndexAlign(
                     stage=OperandStage.map, index_min_max=index_min_max, column_min_max=None,
                     dtype=chunk.dtype, sparse=series.issparse(), output_types=[OutputType.series])
-                out_chunk = align_op.new_chunk([chunk], shape=(np.nan,), index=(out_idx,))
+                params = align_op.build_map_chunk_kw([chunk], shape=(np.nan,), index=(out_idx,))
+                out_chunk = align_op.new_chunk([chunk], **params)
             else:
                 out_chunk = chunk
             out_chunks.append(out_chunk)
@@ -540,7 +535,8 @@ def _gen_series_chunks(splits, out_shape, left_or_right, series):
             map_op = DataFrameIndexAlign(
                 stage=OperandStage.map, sparse=chunk.issparse(), index_shuffle_size=out_shape[0],
                 output_types=[OutputType.series])
-            map_chunks.append(map_op.new_chunk([chunk], shape=(np.nan,), index=chunk.index))
+            params = map_op.build_map_chunk_kw([chunk], shape=(np.nan,), index=chunk.index)
+            map_chunks.append(map_op.new_chunk([chunk], **params))
 
         proxy_chunk = DataFrameShuffleProxy(output_types=[OutputType.series]).new_chunk(
             map_chunks, shape=())
@@ -548,10 +544,11 @@ def _gen_series_chunks(splits, out_shape, left_or_right, series):
         # gen reduce chunks
         for out_idx in range(out_shape[0]):
             reduce_op = DataFrameIndexAlign(stage=OperandStage.reduce, i=out_idx,
-                                            sparse=proxy_chunk.issparse(), shuffle_key=str(out_idx),
+                                            sparse=proxy_chunk.issparse(),
                                             output_types=[OutputType.series])
+            params = reduce_op.build_reduce_chunk_kw([proxy_chunk], index=(out_idx,), shape=(np.nan,))
             out_chunks.append(
-                reduce_op.new_chunk([proxy_chunk], shape=(np.nan,), index=(out_idx,)))
+                reduce_op.new_chunk([proxy_chunk], **params))
 
     return out_chunks
 
@@ -585,7 +582,9 @@ def _gen_dataframe_chunks(splits, out_shape, left_or_right, df):
                     stage=OperandStage.map, index_min_max=index_min_max,
                     column_min_max=column_min_max, dtypes=dtypes, sparse=chunk.issparse(),
                     output_types=[OutputType.dataframe], **kw)
-                out_chunk = align_op.new_chunk([chunk], shape=(np.nan, np.nan), index=out_idx, **chunk_kw)
+                params = align_op.build_map_chunk_kw(
+                    [chunk], shape=(np.nan, np.nan), index=out_idx, **chunk_kw)
+                out_chunk = align_op.new_chunk([chunk], **params)
             else:
                 out_chunk = chunk
             out_chunks.append(out_chunk)
@@ -617,12 +616,15 @@ def _gen_dataframe_chunks(splits, out_shape, left_or_right, df):
                     chunk_kw['index_value'] = input_chunk.index_value if splits[0].isdummy() else None
                 else:
                     chunk_kw['columns_value'] = input_chunk.columns_value if splits[1].isdummy() else None
+                chunk_kw['dtypes'] = input_chunk.dtypes
                 map_op = DataFrameIndexAlign(stage=OperandStage.map, sparse=input_chunk.issparse(),
                                              output_types=[OutputType.dataframe], **kw)
                 idx = [None, None]
                 idx[align_axis] = align_axis_idx
                 idx[shuffle_axis] = j
-                map_chunks.append(map_op.new_chunk([input_chunk], shape=(np.nan, np.nan), index=tuple(idx), **chunk_kw))
+                params = map_op.build_map_chunk_kw([input_chunk], shape=(np.nan, np.nan),
+                                                   index=tuple(idx), **chunk_kw)
+                map_chunks.append(map_op.new_chunk([input_chunk], **params))
             proxy_chunk = DataFrameShuffleProxy(
                 sparse=df.issparse(), output_types=[OutputType.dataframe]).new_chunk(map_chunks, shape=())
             for j in range(out_shape[shuffle_axis]):
@@ -633,12 +635,15 @@ def _gen_dataframe_chunks(splits, out_shape, left_or_right, df):
                 else:
                     chunk_kw['columns_value'] = proxy_chunk.inputs[0].inputs[0].columns_value \
                         if splits[1].isdummy() else None
+                chunk_kw['dtypes'] = proxy_chunk.inputs[0].inputs[0].dtypes
                 reduce_idx = (align_axis_idx, j) if align_axis == 0 else (j, align_axis_idx)
-                reduce_op = DataFrameIndexAlign(stage=OperandStage.reduce, i=j, sparse=proxy_chunk.issparse(),
-                                                shuffle_key=','.join(str(idx) for idx in reduce_idx),
+                reduce_op = DataFrameIndexAlign(stage=OperandStage.reduce, i=j,
+                                                sparse=proxy_chunk.issparse(),
                                                 output_types=[OutputType.dataframe])
+                params = reduce_op.build_reduce_chunk_kw([proxy_chunk], shape=(np.nan, np.nan),
+                                                         index=reduce_idx, **chunk_kw)
                 out_chunks.append(
-                    reduce_op.new_chunk([proxy_chunk], shape=(np.nan, np.nan), index=reduce_idx, **chunk_kw))
+                    reduce_op.new_chunk([proxy_chunk], **params))
         out_chunks.sort(key=lambda c: c.index)
     else:
         # all axes need shuffle
@@ -650,7 +655,8 @@ def _gen_dataframe_chunks(splits, out_shape, left_or_right, df):
             map_op = DataFrameIndexAlign(
                 stage=OperandStage.map, sparse=chunk.issparse(), index_shuffle_size=out_shape[0],
                 column_shuffle_size=out_shape[1], output_types=[OutputType.dataframe])
-            map_chunks.append(map_op.new_chunk([chunk], shape=(np.nan, np.nan), index=chunk.index))
+            params = map_op.build_map_chunk_kw([chunk], shape=(np.nan, np.nan), index=chunk.index)
+            map_chunks.append(map_op.new_chunk([chunk], **params))
 
         proxy_chunk = DataFrameShuffleProxy(output_types=[OutputType.dataframe]).new_chunk(
             map_chunks, shape=())
@@ -659,10 +665,11 @@ def _gen_dataframe_chunks(splits, out_shape, left_or_right, df):
         for out_idx in itertools.product(*(range(s) for s in out_shape)):
             reduce_op = DataFrameIndexAlign(stage=OperandStage.reduce, i=out_idx,
                                             sparse=proxy_chunk.issparse(),
-                                            shuffle_key=','.join(str(idx) for idx in out_idx),
                                             output_types=[OutputType.dataframe])
+            params = reduce_op.build_reduce_chunk_kw([proxy_chunk],
+                                                     index=out_idx, shape=(np.nan, np.nan))
             out_chunks.append(
-                reduce_op.new_chunk([proxy_chunk], shape=(np.nan, np.nan), index=out_idx))
+                reduce_op.new_chunk([proxy_chunk], **params))
 
     return out_chunks
 
@@ -673,22 +680,27 @@ def align_dataframe_dataframe(left, right):
     right_index_chunks = [c.index_value for c in right.cix[:, 0]]
     right_columns_chunks = [c.columns_value for c in right.cix[0, :]]
 
-    index_splits, index_nsplits = _calc_axis_splits(left.index_value, right.index_value,
-                                                    left_index_chunks, right_index_chunks)
-    if _is_index_identical(left_index_chunks, right_index_chunks):
-        index_nsplits = left.nsplits[0]
+    index_splits, index_chunk_shape = _calc_axis_splits(
+        left.index_value, right.index_value, left_index_chunks, right_index_chunks)
 
-    columns_splits, columns_nsplits = _calc_axis_splits(left.columns_value, right.columns_value,
-                                                        left_columns_chunks, right_columns_chunks)
-    if _is_index_identical(left_columns_chunks, right_columns_chunks):
-        columns_nsplits = left.nsplits[1]
+    columns_splits, column_chunk_shape = _calc_axis_splits(
+        left.columns_value, right.columns_value, left_columns_chunks, right_columns_chunks)
 
-    nsplits = [index_nsplits, columns_nsplits]
-    out_chunk_shape = tuple(len(ns) for ns in nsplits)
     splits = _MinMaxSplitInfo(index_splits, columns_splits)
-
+    out_chunk_shape = (len(index_chunk_shape or list(itertools.chain(*index_splits._left_split))),
+                       len(column_chunk_shape or list(itertools.chain(*columns_splits._left_split))))
     left_chunks = _gen_dataframe_chunks(splits, out_chunk_shape, 0, left)
     right_chunks = _gen_dataframe_chunks(splits, out_chunk_shape, 1, right)
+    if _is_index_identical(left_index_chunks, right_index_chunks):
+        index_nsplits = left.nsplits[0]
+    else:
+        index_nsplits = [np.nan for _ in range(out_chunk_shape[0])]
+    if _is_index_identical(left_columns_chunks, right_columns_chunks):
+        columns_nsplits = left.nsplits[1]
+    else:
+        columns_nsplits = [np.nan for _ in range(out_chunk_shape[1])]
+
+    nsplits = [index_nsplits, columns_nsplits]
 
     return nsplits, out_chunk_shape, left_chunks, right_chunks
 
@@ -697,28 +709,40 @@ def align_dataframe_series(left, right, axis='columns'):
     if axis == 'columns' or axis == 1:
         left_columns_chunks = [c.columns_value for c in left.cix[0, :]]
         right_index_chunks = [c.index_value for c in right.chunks]
-        index_splits, index_nsplits = _calc_axis_splits(left.columns_value, right.index_value,
-                                                        left_columns_chunks, right_index_chunks)
+        index_splits, chunk_shape = _calc_axis_splits(left.columns_value, right.index_value,
+                                         left_columns_chunks, right_index_chunks)
+        dummy_splits, dummy_nsplits = _build_dummy_axis_split(
+            left.chunk_shape[0]), left.nsplits[0]
+        out_chunk_shape = (len(dummy_nsplits),
+                           len(chunk_shape or list(itertools.chain(*index_splits._left_split))))
+        left_chunks = _gen_dataframe_chunks(_MinMaxSplitInfo(dummy_splits, index_splits),
+                                            out_chunk_shape, 0, left)
+        right_chunks = _gen_series_chunks(_MinMaxSplitInfo(index_splits, None),
+                                          (out_chunk_shape[1],), 1, right)
         if _is_index_identical(left_columns_chunks, right_index_chunks):
             index_nsplits = left.nsplits[1]
-        dummy_splits, dummy_nsplits = _build_dummy_axis_split(left.chunk_shape[0]), left.nsplits[0]
+        else:
+            index_nsplits = [np.nan for _ in range(out_chunk_shape[1])]
         nsplits = [dummy_nsplits, index_nsplits]
-        out_chunk_shape = tuple(len(ns) for ns in nsplits)
-        left_chunks = _gen_dataframe_chunks(_MinMaxSplitInfo(dummy_splits, index_splits), out_chunk_shape, 0, left)
-        right_chunks = _gen_series_chunks(_MinMaxSplitInfo(index_splits, None), (out_chunk_shape[1],), 1, right)
     else:
         assert axis == 'index' or axis == 0
         left_index_chunks = [c.index_value for c in left.cix[:, 0]]
         right_index_chunks = [c.index_value for c in right.chunks]
-        index_splits, index_nsplits = _calc_axis_splits(left.index_value, right.index_value,
-                                                        left_index_chunks, right_index_chunks)
+        index_splits, index_chunk_shape = _calc_axis_splits(
+            left.index_value, right.index_value, left_index_chunks, right_index_chunks)
+
+        dummy_splits, dummy_nsplits = _build_dummy_axis_split(left.chunk_shape[1]), left.nsplits[1]
+        out_chunk_shape = (len(index_chunk_shape or list(itertools.chain(*index_splits._left_split))),
+                           len(dummy_nsplits))
+        left_chunks = _gen_dataframe_chunks(_MinMaxSplitInfo(index_splits, dummy_splits),
+                                            out_chunk_shape, 0, left)
+        right_chunks = _gen_series_chunks(_MinMaxSplitInfo(index_splits, None),
+                                          (out_chunk_shape[0],), 1, right)
         if _is_index_identical(left_index_chunks, right_index_chunks):
             index_nsplits = left.nsplits[0]
-        dummy_splits, dummy_nsplits = _build_dummy_axis_split(left.chunk_shape[1]), left.nsplits[1]
+        else:
+            index_nsplits = [np.nan for _ in range(out_chunk_shape[0])]
         nsplits = [index_nsplits, dummy_nsplits]
-        out_chunk_shape = tuple(len(ns) for ns in nsplits)
-        left_chunks = _gen_dataframe_chunks(_MinMaxSplitInfo(index_splits, dummy_splits), out_chunk_shape, 0, left)
-        right_chunks = _gen_series_chunks(_MinMaxSplitInfo(index_splits, None), (out_chunk_shape[0],), 1, right)
 
     return nsplits, out_chunk_shape, left_chunks, right_chunks
 
@@ -727,15 +751,17 @@ def align_series_series(left, right):
     left_index_chunks = [c.index_value for c in left.chunks]
     right_index_chunks = [c.index_value for c in right.chunks]
 
-    index_splits, index_nsplits = _calc_axis_splits(left.index_value, right.index_value,
-                                                    left_index_chunks, right_index_chunks)
-    if _is_index_identical(left_index_chunks, right_index_chunks):
-        index_nsplits = left.nsplits[0]
-    nsplits = [index_nsplits]
-    out_chunk_shape = (len(index_nsplits),)
+    index_splits, index_chunk_shape = _calc_axis_splits(
+        left.index_value, right.index_value, left_index_chunks, right_index_chunks)
+
+    out_chunk_shape = (len(index_chunk_shape or list(itertools.chain(*index_splits._left_split))),)
     splits = _MinMaxSplitInfo(index_splits, None)
 
     left_chunks = _gen_series_chunks(splits, out_chunk_shape, 0, left)
     right_chunks = _gen_series_chunks(splits, out_chunk_shape, 1, right)
-
+    if _is_index_identical(left_index_chunks, right_index_chunks):
+        index_nsplits = left.nsplits[0]
+    else:
+        index_nsplits = [np.nan for _ in range(out_chunk_shape[0])]
+    nsplits = [index_nsplits]
     return nsplits, out_chunk_shape, left_chunks, right_chunks
