@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,8 +14,7 @@
 
 import functools
 
-from ...utils import is_build_mode
-from ..core import DATAFRAME_TYPE
+from ..core import DATAFRAME_TYPE, is_build_mode
 from ..utils import wrap_notimplemented_exception
 from ..ufunc.tensor import register_tensor_ufunc
 from .abs import abs_, DataFrameAbs
