@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,11 +16,11 @@ import operator
 
 from ... import opcodes as OperandDef
 from ...utils import classproperty
-from .core import DataFrameBinOpMixin, DataFrameBinOp
+from .core import DataFrameBinopUfunc
 from .docstring import bin_arithmetic_doc
 
 
-class DataFrameTrueDiv(DataFrameBinOp, DataFrameBinOpMixin):
+class DataFrameTrueDiv(DataFrameBinopUfunc):
     _op_type_ = OperandDef.DIV
 
     _func_name = 'truediv'
@@ -30,6 +30,11 @@ class DataFrameTrueDiv(DataFrameBinOp, DataFrameBinOpMixin):
     def _operator(self):
         return operator.truediv
 
+    @classproperty
+    def tensor_op_type(self):
+        from ...tensor.arithmetic import TensorTrueDiv
+        return TensorTrueDiv
+
 
 _truediv_example = """
 >>> a.truediv(b, fill_value=0).execute()
