@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,14 +19,14 @@ import pandas as pd
 
 from ... import opcodes
 from ...config import options
-from ...core import OutputType
-from ...custom_log import redirect_custom_log
-from ...serialize import StringField, AnyField, BoolField, \
+from ...core import OutputType, recursive_tile
+from ...core.custom_log import redirect_custom_log
+from ...serialization.serializables import StringField, AnyField, BoolField, \
     TupleField, DictField, FunctionField
 from ...utils import enter_current_session, quiet_stdio
 from ..operands import DataFrameOperandMixin, DataFrameOperand
 from ..utils import build_df, build_series, parse_index, validate_axis, \
-    validate_output_types, make_dtypes, make_dtype
+    validate_output_types, make_dtypes, make_dtype, build_empty_df, build_empty_series
 
 
 class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
@@ -40,17 +40,14 @@ class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
     _elementwise = BoolField('elementwise')
     _args = TupleField('args')
     _kwds = DictField('kwds')
-    # for chunk
-    _tileable_op_key = StringField('tileable_op_key')
 
     def __init__(self, func=None, axis=None, convert_dtype=None, raw=None, result_type=None,
-                 args=None, kwds=None, output_type=None, elementwise=None,
-                 tileable_op_key=None, **kw):
+                 args=None, kwds=None, output_type=None, elementwise=None, **kw):
         if output_type:
             kw['_output_types'] = [output_type]
         super().__init__(_func=func, _axis=axis, _convert_dtype=convert_dtype, _raw=raw,
                          _result_type=result_type, _args=args, _kwds=kwds,
-                         _elementwise=elementwise, _tileable_op_key=tileable_op_key, **kw)
+                         _elementwise=elementwise, **kw)
 
     @property
     def func(self):
@@ -84,22 +81,27 @@ class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
     def kwds(self):
         return getattr(self, '_kwds', None) or dict()
 
-    @property
-    def tileable_op_key(self):
-        return self._tileable_op_key
-
     @classmethod
     @redirect_custom_log
     @enter_current_session
     def execute(cls, ctx, op):
         input_data = ctx[op.inputs[0].key]
+        out = op.outputs[0]
+        if len(input_data) == 0:
+            if op.output_types[0] == OutputType.dataframe:
+                ctx[out.key] = build_empty_df(out.dtypes)
+            else:
+                ctx[out.key] = build_empty_series(
+                    out.dtype, name=out.name)
+            return
+
         if isinstance(input_data, pd.DataFrame):
             result = input_data.apply(op.func, axis=op.axis, raw=op.raw, result_type=op.result_type,
                                       args=op.args, **op.kwds)
         else:
             result = input_data.apply(op.func, convert_dtype=op.convert_dtype, args=op.args,
                                       **op.kwds)
-        ctx[op.outputs[0].key] = result
+        ctx[out.key] = result
 
     @classmethod
     def _tile_df(cls, op):
@@ -115,7 +117,7 @@ class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
             )
             if axis == 1:
                 chunk_size = chunk_size[::-1]
-            in_df = in_df.rechunk(chunk_size)._inplace_tile()
+            in_df = yield from recursive_tile(in_df.rechunk(chunk_size))
 
         chunks = []
         if out_df.ndim == 2:
@@ -139,7 +141,7 @@ class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
                     new_dtypes = out_df.dtypes
 
                 new_op = op.copy().reset_key()
-                new_op._tileable_op_key = op.key
+                new_op.tileable_op_key = op.key
                 chunks.append(new_op.new_chunk([c], shape=tuple(new_shape), index=c.index, dtypes=new_dtypes,
                                                index_value=new_index_value, columns_value=new_columns_value))
 
@@ -152,7 +154,7 @@ class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
                 new_index_value = c.index_value if axis == 1 else c.columns_value
                 new_index = (c.index[1 - axis],)
                 new_op = op.copy().reset_key()
-                new_op._tileable_op_key = op.key
+                new_op.tileable_op_key = op.key
                 chunks.append(new_op.new_chunk([c], shape=(shape_len,), index=new_index, dtype=out_df.dtype,
                                                index_value=new_index_value))
             new_nsplits = (in_df.nsplits[1 - axis],)
@@ -170,7 +172,7 @@ class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
         chunks = []
         for c in in_series.chunks:
             new_op = op.copy().reset_key()
-            new_op._tileable_op_key = op.key
+            new_op.tileable_op_key = op.key
             kw = c.params.copy()
             if out_series.ndim == 1:
                 kw['dtype'] = out_series.dtype
@@ -192,7 +194,7 @@ class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
     @classmethod
     def tile(cls, op):
         if op.inputs[0].ndim == 2:
-            return cls._tile_df(op)
+            return (yield from cls._tile_df(op))
         else:
             return cls._tile_series(op)
 
@@ -257,10 +259,10 @@ class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
         if self.output_types[0] == OutputType.dataframe:
             if self.axis == 0:
                 return self.new_dataframe([df], shape=shape, dtypes=dtypes, index_value=index_value,
-                                          columns_value=parse_index(dtypes.index))
+                                          columns_value=parse_index(dtypes.index, store_data=True))
             else:
                 return self.new_dataframe([df], shape=shape, dtypes=dtypes, index_value=df.index_value,
-                                          columns_value=parse_index(dtypes.index))
+                                          columns_value=parse_index(dtypes.index, store_data=True))
         else:
             name, dtype = dtypes
             return self.new_series([df], shape=shape, name=name, dtype=dtype, index_value=index_value)
@@ -292,7 +294,6 @@ class ApplyOperand(DataFrameOperand, DataFrameOperandMixin):
                     else:
                         raise TypeError('Cannot determine dtypes, '
                                         'please specify `dtypes` as argument')
-
                 columns_value = parse_index(dtypes.index, store_data=True)
 
                 return self.new_dataframe([series], shape=(series.shape[0], len(dtypes)),
