@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,9 +17,10 @@ import pandas as pd
 
 from ... import opcodes as OperandDef
 from ... import tensor as mt
-from ...operands import OperandStage
-from ...serialize import ValueType, KeyField, ListField
-from ...utils import recursive_tile, lazy_import
+from ...core import recursive_tile
+from ...core.operand import OperandStage
+from ...serialization.serializables import FieldTypes, KeyField, ListField, AnyField
+from ...utils import lazy_import
 from ..core import SERIES_TYPE
 from ..initializer import DataFrame, Series
 from ..operands import DataFrameOperand, DataFrameOperandMixin
@@ -33,15 +34,14 @@ class DataFrameDescribe(DataFrameOperand, DataFrameOperandMixin):
     _op_type_ = OperandDef.DESCRIBE
 
     _input = KeyField('input')
-    _percentiles = ListField('percentiles', ValueType.float64)
-    _include = ListField('include')
-    _exclude = ListField('exclude')
+    _percentiles = ListField('percentiles', FieldTypes.float64)
+    _include = AnyField('include')
+    _exclude = AnyField('exclude')
 
     def __init__(self, percentiles=None, include=None, exclude=None,
-                 output_types=None, stage=None, **kw):
+                 output_types=None, **kw):
         super().__init__(_percentiles=percentiles, _include=include,
-                         _exclude=exclude, _output_types=output_types,
-                         _stage=stage, **kw)
+                         _exclude=exclude, _output_types=output_types, **kw)
 
     @property
     def input(self):
@@ -61,7 +61,7 @@ class DataFrameDescribe(DataFrameOperand, DataFrameOperandMixin):
 
     def _set_inputs(self, inputs):
         super()._set_inputs(inputs)
-        if self._stage != OperandStage.agg:
+        if self.stage != OperandStage.agg:
             self._input = self._inputs[0]
 
     def __call__(self, df_or_series):
@@ -98,9 +98,10 @@ class DataFrameDescribe(DataFrameOperand, DataFrameOperandMixin):
             return cls._tile_one_chunk(op)
 
         if isinstance(inp, SERIES_TYPE):
-            return cls._tile_series(op)
+            result = yield from cls._tile_series(op)
         else:
-            return cls._tile_dataframe(op)
+            result = yield from cls._tile_dataframe(op)
+        return result
 
     @classmethod
     def _tile_one_chunk(cls, op):
@@ -133,7 +134,8 @@ class DataFrameDescribe(DataFrameOperand, DataFrameOperandMixin):
 
         t = mt.concatenate(values).rechunk(len(names))
         ret = Series(t, index=index, name=series.name)
-        return [recursive_tile(ret)]
+        ret = yield from recursive_tile(ret)
+        return [ret]
 
     @classmethod
     def _tile_dataframe(cls, op):
@@ -143,7 +145,7 @@ class DataFrameDescribe(DataFrameOperand, DataFrameOperandMixin):
         columns = dtypes.index.tolist()
 
         if df.chunk_shape[1] > 1:
-            df = df.rechunk({1: df.shape[1]})._inplace_tile()
+            df = df.rechunk({1: df.shape[1]})
 
         # check dtypes if selected all fields
         # to reduce graph scale
@@ -151,11 +153,13 @@ class DataFrameDescribe(DataFrameOperand, DataFrameOperandMixin):
             df = df[columns]
 
         # perform aggregation together
-        aggregation = recursive_tile(df.agg(['count', 'mean', 'std', 'min', 'max']))
+        aggregation = yield from recursive_tile(
+            df.agg(['count', 'mean', 'std', 'min', 'max']))
         # calculate percentiles
         percentiles = None
         if len(op.percentiles) > 0:
-            percentiles = recursive_tile(df.quantile(op.percentiles))
+            percentiles = yield from recursive_tile(
+                df.quantile(op.percentiles))
 
         chunk_op = DataFrameDescribe(output_types=op.output_types,
                                      stage=OperandStage.agg,
@@ -186,7 +190,7 @@ class DataFrameDescribe(DataFrameOperand, DataFrameOperandMixin):
 
             inputs = [ctx[inp.key] for inp in op.inputs]
             xdf = pd if isinstance(inputs[0], (pd.DataFrame, pd.Series, pd.Index)) \
-                        or cudf is None else cudf
+                or cudf is None else cudf
 
             if len(inputs) == 1:
                 df = inputs[0]
@@ -222,7 +226,7 @@ def describe(df_or_series, percentiles=None, include=None, exclude=None):
         unique_pcts = np.unique(percentiles)
         if len(unique_pcts) < len(percentiles):
             raise ValueError("percentiles cannot contain duplicates")
-        percentiles = unique_pcts
+        percentiles = unique_pcts.tolist()
 
     op = DataFrameDescribe(percentiles=percentiles, include=include, exclude=exclude)
     return op(df_or_series)
