@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,11 +18,10 @@ import pandas as pd
 
 from ... import opcodes as OperandDef
 from ...core import OutputType
-from ...serialize import KeyField, AnyField, Int32Field, Int64Field
+from ...serialization.serializables import KeyField, AnyField, Int32Field, Int64Field
 from ...tensor.rechunk.core import get_nsplits, plan_rechunks, compute_rechunk_slices
 from ...tensor.utils import calc_sliced_size
-from ...tiles import TilesError
-from ...utils import check_chunks_unknown_shape
+from ...utils import has_unknown_shape
 from ..initializer import DataFrame as asdataframe, Series as asseries
 from ..operands import DataFrameOperand, DataFrameOperandMixin, DATAFRAME_TYPE
 from ..utils import indexing_index_value, merge_index_value
@@ -36,11 +35,9 @@ class DataFrameRechunk(DataFrameOperand, DataFrameOperandMixin):
     _threshold = Int32Field('threshold')
     _chunk_size_limit = Int64Field('chunk_size_limit')
 
-    def __init__(self, chunk_size=None, threshold=None, chunk_size_limit=None, output_types=None,
-                 reassign_worker=None, **kw):
+    def __init__(self, chunk_size=None, threshold=None, chunk_size_limit=None, output_types=None, **kw):
         super().__init__(_chunk_size=chunk_size, _threshold=threshold,
-                         _chunk_size_limit=chunk_size_limit, _output_types=output_types,
-                         _reassign_worker=reassign_worker, **kw)
+                         _chunk_size_limit=chunk_size_limit, _output_types=output_types, **kw)
 
     @property
     def input(self):
@@ -73,7 +70,8 @@ class DataFrameRechunk(DataFrameOperand, DataFrameOperandMixin):
 
     @classmethod
     def tile(cls, op):
-        check_chunks_unknown_shape(op.inputs, TilesError)
+        if has_unknown_shape(*op.inputs):
+            yield
 
         a = op.input
         a = asdataframe(a) if a.ndim == 2 else asseries(a)
@@ -96,7 +94,7 @@ class DataFrameRechunk(DataFrameOperand, DataFrameOperandMixin):
 
         if op.reassign_worker:
             for c in out.chunks:
-                c.op._reassign_worker = True
+                c.op.reassign_worker = True
 
         return [out]
 
@@ -111,13 +109,7 @@ def _get_chunk_size(a, chunk_size):
 
 def rechunk(a, chunk_size, threshold=None, chunk_size_limit=None, reassign_worker=False):
     if not any(pd.isna(s) for s in a.shape) and not a.is_coarse():
-        try:
-            check_chunks_unknown_shape([a], ValueError)
-        except ValueError:
-            # due to reason that tileable has unknown chunk shape,
-            # just ignore to hand over to operand
-            pass
-        else:
+        if not has_unknown_shape(a):
             # do client check only when no unknown shape,
             # real nsplits will be recalculated inside `tile`
             chunk_size = _get_chunk_size(a, chunk_size)
@@ -174,14 +166,14 @@ def compute_rechunk(a, chunk_size):
             new_index_value = indexing_index_value(old_chunk.index_value, chunk_slice[0])
             if is_dataframe:
                 new_columns_value = indexing_index_value(old_chunk.columns_value, chunk_slice[1], store_data=True)
-                merge_chunk_op = DataFrameIlocGetItem(chunk_slice, sparse=old_chunk.op.sparse,
+                merge_chunk_op = DataFrameIlocGetItem(list(chunk_slice), sparse=old_chunk.op.sparse,
                                                       output_types=[OutputType.dataframe])
                 merge_chunk = merge_chunk_op.new_chunk([old_chunk], shape=merge_chunk_shape,
                                                        index=merge_idx, index_value=new_index_value,
                                                        columns_value=new_columns_value,
                                                        dtypes=old_chunk.dtypes.iloc[chunk_slice[1]])
             else:
-                merge_chunk_op = SeriesIlocGetItem(chunk_slice, sparse=old_chunk.op.sparse,
+                merge_chunk_op = SeriesIlocGetItem(list(chunk_slice), sparse=old_chunk.op.sparse,
                                                    output_types=a.op.output_types)
                 merge_chunk = merge_chunk_op.new_chunk([old_chunk], shape=merge_chunk_shape,
                                                        index=merge_idx, index_value=new_index_value,
