@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,13 +17,12 @@ import pandas as pd
 
 from ... import opcodes as OperandDef
 from ...core import OutputType
-from ...serialize import KeyField, AnyField, Int8Field, Int64Field
-from ...tiles import TilesError
-from ...utils import check_chunks_unknown_shape
+from ...serialization.serializables import KeyField, AnyField, Int8Field, Int64Field
+from ...utils import has_unknown_shape
 from ..operands import DataFrameOperand, DataFrameOperandMixin
 from ..utils import parse_index, build_df, build_series, validate_axis
 
-_need_consolidate = pd.__version__ in ('1.1.0', '1.3.0')
+_need_consolidate = pd.__version__ in ('1.1.0', '1.3.0', '1.3.1')
 
 
 class DataFrameShift(DataFrameOperand, DataFrameOperandMixin):
@@ -160,7 +159,7 @@ class DataFrameShift(DataFrameOperand, DataFrameOperandMixin):
                 out_chunks.append(out_chunk)
         else:
             if np.isnan(np.sum(inp.nsplits[axis])):  # pragma: no cover
-                raise TilesError(f'input has unknown chunk shape on axis {axis}')
+                yield
 
             # shift data
             inc = op.periods > 0
@@ -212,7 +211,7 @@ class DataFrameShift(DataFrameOperand, DataFrameOperandMixin):
 
                     chunk_op = op.copy().reset_key()
                     if axis == 1:
-                        dtypes = c.dtypes.iloc[start: end]
+                        dtypes = out.dtypes.iloc[start: end]
                         columns_value = parse_index(dtypes.index, store_data=True)
                         index_value = c.index_value
                     else:
@@ -238,7 +237,8 @@ class DataFrameShift(DataFrameOperand, DataFrameOperandMixin):
         from ..indexing.iloc import SeriesIlocGetItem
         from ..merge import DataFrameConcat
 
-        check_chunks_unknown_shape(op.inputs, TilesError)
+        if has_unknown_shape(*op.inputs):
+            yield
 
         inp = op.input
         out = op.outputs[0]
@@ -302,9 +302,9 @@ class DataFrameShift(DataFrameOperand, DataFrameOperandMixin):
     @classmethod
     def tile(cls, op):
         if op.output_types[0] == OutputType.dataframe:
-            return cls._tile_dataframe(op)
+            return (yield from cls._tile_dataframe(op))
         else:
-            return cls._tile_series(op)
+            return (yield from cls._tile_series(op))
 
     @classmethod
     def execute(cls, ctx, op):
