@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,16 +12,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from distutils.version import LooseVersion
-
 import numpy as np
 import pandas as pd
 
 from ... import opcodes
 from ...config import options
-from ...core import OutputType
-from ...custom_log import redirect_custom_log
-from ...serialize import AnyField, BoolField, TupleField, DictField, StringField
+from ...core import OutputType, recursive_tile
+from ...core.custom_log import redirect_custom_log
+from ...lib.version import parse as parse_version
+from ...serialization.serializables import AnyField, BoolField, \
+    TupleField, DictField
 from ...utils import enter_current_session, quiet_stdio
 from ..core import DATAFRAME_CHUNK_TYPE, DATAFRAME_TYPE
 from ..operands import DataFrameOperandMixin, DataFrameOperand
@@ -37,16 +37,14 @@ class TransformOperand(DataFrameOperand, DataFrameOperandMixin):
     _convert_dtype = BoolField('convert_dtype')
     _args = TupleField('args')
     _kwds = DictField('kwds')
-    # for chunk
-    _tileable_op_key = StringField('tileable_op_key')
 
     _call_agg = BoolField('call_agg')
 
     def __init__(self, func=None, axis=None, convert_dtype=None, args=None, kwds=None,
-                 call_agg=None, output_types=None, tileable_op_key=None, memory_scale=None, **kw):
+                 call_agg=None, output_types=None, memory_scale=None, **kw):
         super().__init__(_func=func, _axis=axis, _convert_dtype=convert_dtype, _args=args,
                          _kwds=kwds, _call_agg=call_agg, _output_types=output_types,
-                         _tileable_op_key=tileable_op_key, _memory_scale=memory_scale, **kw)
+                         _memory_scale=memory_scale, **kw)
 
     @property
     def func(self):
@@ -72,10 +70,6 @@ class TransformOperand(DataFrameOperand, DataFrameOperandMixin):
     def call_agg(self):
         return self._call_agg
 
-    @property
-    def tileable_op_key(self):
-        return self._tileable_op_key
-
     @classmethod
     @redirect_custom_log
     @enter_current_session
@@ -104,19 +98,19 @@ class TransformOperand(DataFrameOperand, DataFrameOperandMixin):
                               max(1, options.chunk_store_limit // in_df.shape[axis]))
                 if axis == 1:
                     chunk_size = chunk_size[::-1]
-                in_df = in_df.rechunk(chunk_size)._inplace_tile()
+                in_df = yield from recursive_tile(in_df.rechunk(chunk_size))
         elif isinstance(op.func, str) or \
-                 (isinstance(op.func, list) and any(isinstance(e, str) for e in op.func)):
+                (isinstance(op.func, list) and any(isinstance(e, str) for e in op.func)):
             # builtin cols handles whole columns, thus merge is needed
             if in_df.chunk_shape[0] > 1:
-                in_df = in_df.rechunk((in_df.shape[axis],))._inplace_tile()
+                in_df = yield from recursive_tile(in_df.rechunk((in_df.shape[axis],)))
 
         chunks = []
         axis_index_map = dict()
         col_sizes = []
         for c in in_df.chunks:
             new_op = op.copy().reset_key()
-            new_op._tileable_op_key = op.key
+            new_op.tileable_op_key = op.key
             params = c.params.copy()
 
             if out_df.ndim == 2:
@@ -201,7 +195,7 @@ class TransformOperand(DataFrameOperand, DataFrameOperandMixin):
                     if self.call_agg:
                         infer_df = test_df.agg(self._func, args=self.args, **self.kwds)
                     else:
-                        if LooseVersion(pd.__version__) >= '1.2.0':
+                        if parse_version(pd.__version__) >= parse_version('1.2.0'):
                             infer_df = test_df.transform(self._func, *self.args, **self.kwds)
                         else:  # pragma: no cover
                             infer_df = test_df.transform(self._func, convert_dtype=self.convert_dtype,
