@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,12 +20,11 @@ import numpy as np
 import pandas as pd
 
 from ... import opcodes as OperandDef
-from ...core import Base, Entity, OutputType
-from ...serialize import KeyField, SeriesField, DataTypeField, AnyField
+from ...core import ENTITY_TYPE, OutputType, recursive_tile
+from ...serialization.serializables import KeyField, SeriesField, DataTypeField, AnyField
 from ...tensor.datasource import tensor as astensor
 from ...tensor.utils import unify_chunks
-from ...tiles import TilesError
-from ...utils import check_chunks_unknown_shape
+from ...utils import has_unknown_shape
 from ..core import INDEX_TYPE, SERIES_TYPE, SERIES_CHUNK_TYPE
 from ..operands import DataFrameOperand, DataFrameOperandMixin
 from ..utils import parse_index
@@ -67,7 +66,7 @@ class DataFrameFromTensor(DataFrameOperand, DataFrameOperandMixin):
                 # check each value for input
                 new_input = OrderedDict()
                 for k, v in self._input.items():
-                    if isinstance(v, (Base, Entity)):
+                    if isinstance(v, ENTITY_TYPE):
                         new_input[k] = next(inputs_iter)
                     else:
                         new_input[k] = v
@@ -90,7 +89,7 @@ class DataFrameFromTensor(DataFrameOperand, DataFrameOperandMixin):
                 self._index = index
                 index_value = index.index_value
                 inputs.append(index)
-            elif isinstance(index, (Base, Entity)):
+            elif isinstance(index, ENTITY_TYPE):
                 self._index = index
                 index = astensor(index)
                 if index.ndim != 1:
@@ -115,7 +114,7 @@ class DataFrameFromTensor(DataFrameOperand, DataFrameOperandMixin):
                 elif shape != tileable_shape:
                     raise ValueError('input 1-d tensors should have same shape')
 
-            if isinstance(tileable, (Base, Entity)):
+            if isinstance(tileable, ENTITY_TYPE):
                 tileables.append(tileable)
 
         if index is not None:
@@ -138,7 +137,7 @@ class DataFrameFromTensor(DataFrameOperand, DataFrameOperandMixin):
                 raise ValueError(
                     f'columns {columns} should have size {len(input_1d_tileables)}')
             if not isinstance(columns, pd.Index):
-                if isinstance(columns, Base):
+                if isinstance(columns, ENTITY_TYPE):
                     raise NotImplementedError('The columns value cannot be a tileable')
                 columns = pd.Index(columns)
             columns_value = parse_index(columns, store_data=True)
@@ -172,7 +171,7 @@ class DataFrameFromTensor(DataFrameOperand, DataFrameOperandMixin):
                 raise ValueError(
                     f'columns {columns} should have the same shape with tensor: {input_tensor.shape[1]}')
             if not isinstance(columns, pd.Index):
-                if isinstance(columns, Base):
+                if isinstance(columns, ENTITY_TYPE):
                     raise NotImplementedError('The columns value cannot be a tileable')
                 columns = pd.Index(columns)
             columns_value = parse_index(columns, store_data=True)
@@ -204,7 +203,7 @@ class DataFrameFromTensor(DataFrameOperand, DataFrameOperandMixin):
 
         if columns is not None:
             if not isinstance(columns, pd.Index):
-                if isinstance(columns, Base):
+                if isinstance(columns, ENTITY_TYPE):
                     raise NotImplementedError('The columns value cannot be a tileable')
                 columns = pd.Index(columns)
             columns_value = parse_index(columns, store_data=True)
@@ -219,20 +218,21 @@ class DataFrameFromTensor(DataFrameOperand, DataFrameOperandMixin):
     @classmethod
     def tile(cls, op):
         if isinstance(op.input, dict):
-            return cls._tile_input_1d_tileables(op)
+            return (yield from cls._tile_input_1d_tileables(op))
         elif op.input is not None:
-            return cls._tile_input_tensor(op)
+            return (yield from cls._tile_input_tensor(op))
         else:
             return cls._tile_tensor_none(op)
 
     @classmethod
     def _tile_input_1d_tileables(cls, op):
         # make sure all tensor have known chunk shapes
-        check_chunks_unknown_shape(op.inputs, TilesError)
+        if has_unknown_shape(*op.inputs):
+            yield
 
         out_df = op.outputs[0]
         in_tensors = op.inputs
-        in_tensors = unify_chunks(*in_tensors)
+        in_tensors = yield from unify_chunks(*in_tensors)
         nsplit = in_tensors[0].nsplits[0]
 
         cum_sizes = [0] + np.cumsum(nsplit).tolist()
@@ -241,7 +241,7 @@ class DataFrameFromTensor(DataFrameOperand, DataFrameOperandMixin):
             chunk_op = op.copy().reset_key()
             new_input = OrderedDict()
             for k, v in op.input.items():
-                if not isinstance(v, (Base, Entity)):
+                if not isinstance(v, ENTITY_TYPE):
                     try:
                         new_input[k] = v[cum_sizes[i]: cum_sizes[i + 1]]
                     except TypeError:
@@ -283,12 +283,17 @@ class DataFrameFromTensor(DataFrameOperand, DataFrameOperandMixin):
         out_df = op.outputs[0]
         in_tensor = op.input
         out_chunks = []
+        if out_df.index_value.has_value() and has_unknown_shape(in_tensor):
+            yield
+
         nsplits = in_tensor.nsplits
 
         if op.index is not None:
             # rechunk index if it's a tensor
-            check_chunks_unknown_shape(op.inputs, TilesError)
-            index_tensor = op.index.rechunk([nsplits[0]])._inplace_tile()
+            if has_unknown_shape(*op.inputs):
+                yield
+            index_tensor = yield from recursive_tile(
+                op.index.rechunk([nsplits[0]]))
         else:
             index_tensor = None
 
@@ -487,7 +492,8 @@ class SeriesFromTensor(DataFrameOperand, DataFrameOperandMixin):
     def tile(cls, op):
         if op.index is None:
             # check all inputs to make sure no unknown chunk shape
-            check_chunks_unknown_shape(op.inputs, TilesError)
+            if has_unknown_shape(*op.inputs):
+                yield
 
         if op.input is None:
             return cls._tile_tensor_none(op)
@@ -497,7 +503,8 @@ class SeriesFromTensor(DataFrameOperand, DataFrameOperandMixin):
         nsplits = in_tensor.nsplits
 
         if op.index is not None:
-            index_tensor = op.index.rechunk([nsplits[0]])._inplace_tile()
+            index_tensor = yield from recursive_tile(
+                op.index.rechunk([nsplits[0]]))
         else:
             index_tensor = None
 
@@ -579,7 +586,7 @@ class SeriesFromTensor(DataFrameOperand, DataFrameOperandMixin):
                     self._index = index
                     index_value = index.index_value
                     inputs.append(index)
-                elif isinstance(index, (Base, Entity)):
+                elif isinstance(index, ENTITY_TYPE):
                     self._index = index
                     index = astensor(index)
                     if index.ndim != 1:
