@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -35,14 +35,14 @@ except ImportError:
 from ... import opcodes as OperandDef
 from ...config import options
 from ...lib.filesystem import file_size, get_fs, glob, open_file
-from ...serialize import AnyField, BoolField, DictField, ListField,\
+from ...serialization.serializables import AnyField, BoolField, DictField, ListField,\
     StringField, Int32Field, Int64Field, BytesField
 from ...utils import is_object_dtype
 from ..arrays import ArrowStringDtype
 from ..operands import OutputType
-from ..utils import parse_index, to_arrow_dtypes, contain_arrow_dtype, \
-    standardize_range_index
-from .core import HeadOptimizedDataSource, ColumnPruneSupportedDataSourceMixin
+from ..utils import parse_index, to_arrow_dtypes, contain_arrow_dtype
+from .core import IncrementalIndexDatasource, ColumnPruneSupportedDataSourceMixin, \
+    IncrementalIndexDataSourceMixin
 
 
 PARQUET_MEMORY_SCALE = 15
@@ -59,7 +59,7 @@ def check_engine(engine):
             raise RuntimeError('Please install either pyarrow or fastparquet.')
     elif engine == 'pyarrow':
         if pa is None:  # pragma: no cover
-            raise RuntimeError('Please install pyarrow fisrt.')
+            raise RuntimeError('Please install pyarrow first.')
         return engine
     elif engine == 'fastparquet':
         if fastparquet is None:  # pragma: no cover
@@ -151,7 +151,9 @@ class FastpaquetEngine(ParquetEngine):
         return df
 
 
-class DataFrameReadParquet(HeadOptimizedDataSource, ColumnPruneSupportedDataSourceMixin):
+class DataFrameReadParquet(IncrementalIndexDatasource,
+                           ColumnPruneSupportedDataSourceMixin,
+                           IncrementalIndexDataSourceMixin):
     _op_type_ = OperandDef.READ_PARQUET
 
     _path = AnyField('path')
@@ -353,9 +355,6 @@ class DataFrameReadParquet(HeadOptimizedDataSource, ColumnPruneSupportedDataSour
                 out_chunks.append(new_chunk)
                 chunk_index += 1
 
-        if op.incremental_index:
-            out_chunks = standardize_range_index(out_chunks)
-
         new_op = op.copy()
         nsplits = ((np.nan,) * len(out_chunks), (out_df.shape[1],))
         return new_op.new_dataframes(None, out_df.shape, dtypes=dtypes,
@@ -411,7 +410,7 @@ class DataFrameReadParquet(HeadOptimizedDataSource, ColumnPruneSupportedDataSour
         first_chunk_raw_bytes = op.first_chunk_raw_bytes
         raw_bytes = file_size(op.path, storage_options=op.storage_options)
         if op.num_group_rows:
-            raw_bytes = np.ceil(np.divide(raw_bytes, op.num_row_groups)) \
+            raw_bytes = np.ceil(np.divide(raw_bytes, op.num_group_rows)) \
                 .astype(np.int64).item()
 
         estimated_row_num = np.ceil(
@@ -458,7 +457,8 @@ def read_parquet(path, engine: str = "auto", columns=None,
         if False, each file correspond to a chunk.
         Only available for 'pyarrow' engine.
     incremental_index: bool, default False
-        Create a new RangeIndex if csv doesn't contain index columns.
+        If index_col not specified, ensure range index incremental,
+        gain a slightly better performance if setting False.
     use_arrow_dtype: bool, default None
         If True, use arrow dtype to store columns.
     storage_options: dict, optional
