@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,10 +18,10 @@ import numpy as np
 import pandas as pd
 
 from ... import opcodes as OperandDef
-from ...core import OutputType
+from ...core import OutputType, recursive_tile
+from ...core.operand import OperandStage
 from ...lib.filesystem import open_file
-from ...operands import OperandStage
-from ...serialize import KeyField, AnyField, StringField, ListField, \
+from ...serialization.serializables import KeyField, AnyField, StringField, ListField, \
     BoolField, Int32Field, Int64Field, DictField
 from ...tensor.core import TensorOrder
 from ...tensor.operands import TensorOperand, TensorOperandMixin
@@ -61,14 +61,14 @@ class DataFrameToCSV(DataFrameOperand, DataFrameOperandMixin):
                  mode=None, encoding=None, compression=None, quoting=None,
                  quotechar=None, line_terminator=None, chunksize=None, date_format=None,
                  doublequote=None, escapechar=None, decimal=None, output_stat=None,
-                 storage_options=None, stage=None, output_types=None, **kw):
+                 storage_options=None, output_types=None, **kw):
         super().__init__(_path=path, _sep=sep, _na_rep=na_rep, _float_format=float_format,
                          _columns=columns, _header=header, _index=index, _index_label=index_label,
                          _mode=mode, _encoding=encoding, _compression=compression, _quoting=quoting,
                          _quotechar=quotechar, _line_terminator=line_terminator, _chunksize=chunksize,
                          _date_format=date_format, _doublequote=doublequote,
                          _escapechar=escapechar, _decimal=decimal, _output_stat=output_stat,
-                         _storage_options=storage_options, _output_types=output_types, _stage=stage, **kw)
+                         _storage_options=storage_options, _output_types=output_types, **kw)
 
     @property
     def input(self):
@@ -178,7 +178,7 @@ class DataFrameToCSV(DataFrameOperand, DataFrameOperandMixin):
 
         if in_df.ndim == 2 and in_df.chunk_shape[1] > 1:
             # make sure only 1 chunk on the column axis
-            in_df = in_df.rechunk({1: in_df.shape[1]})._inplace_tile()
+            in_df = yield from recursive_tile(in_df.rechunk({1: in_df.shape[1]}))
 
         one_file = op.one_file
 
@@ -201,7 +201,8 @@ class DataFrameToCSV(DataFrameOperand, DataFrameOperandMixin):
                 out_chunks[0].append(out_chunk)
             else:
                 chunk_op._output_stat = True
-                chunk_op._stage = OperandStage.map
+                chunk_op.stage = OperandStage.map
+                chunk_op.output_types = [OutputType.scalar] * 2
                 # bytes of csv
                 kws = [{
                     'shape': (),
