@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,6 +14,7 @@
 
 import functools
 import itertools
+import uuid
 from typing import List, Dict
 
 import numpy as np
@@ -21,11 +22,13 @@ import pandas as pd
 
 from ... import opcodes as OperandDef
 from ...config import options
-from ...context import get_context, RunningMode
-from ...core import Base, Entity, OutputType
-from ...custom_log import redirect_custom_log
-from ...operands import OperandStage
-from ...serialize import Int32Field, AnyField, BoolField, StringField, ListField, DictField
+from ...core.custom_log import redirect_custom_log
+from ...core import ENTITY_TYPE, OutputType
+from ...core.context import get_context
+from ...core.operand import OperandStage
+from ...serialization.serializables import Int32Field, AnyField, BoolField, \
+    StringField, ListField, DictField
+from ...typing import ChunkType, TileableType
 from ...utils import enter_current_session, lazy_import
 from ..core import GROUPBY_TYPE
 from ..merge import DataFrameConcat
@@ -33,13 +36,28 @@ from ..operands import DataFrameOperand, DataFrameOperandMixin, DataFrameShuffle
 from ..reduction.core import ReductionCompiler, ReductionSteps, ReductionPreStep, \
     ReductionAggStep, ReductionPostStep
 from ..reduction.aggregation import is_funcs_aggregate, normalize_reduction_funcs
-from ..utils import parse_index, build_concatenated_rows_frame
+from ..utils import parse_index, build_concatenated_rows_frame, is_cudf
 from .core import DataFrameGroupByOperand
 
 cp = lazy_import('cupy', globals=globals(), rename='cp')
 cudf = lazy_import('cudf', globals=globals())
 
 
+class SizeRecorder:
+    def __init__(self):
+        self._raw_records = 0
+        self._agg_records = 0
+
+    def record(self,
+               raw_records: int,
+               agg_records: int):
+        self._raw_records += raw_records
+        self._agg_records += agg_records
+
+    def get(self):
+        return self._raw_records, self._agg_records
+
+
 _agg_functions = {
     'sum': lambda x: x.sum(),
     'prod': lambda x: x.prod(),
@@ -66,7 +84,10 @@ def _patch_groupby_kurt():
         from pandas.core.groupby import DataFrameGroupBy, SeriesGroupBy
         if not hasattr(DataFrameGroupBy, 'kurt'):  # pragma: no branch
             def _kurt_by_frame(a, *args, **kwargs):
-                return a.to_frame().kurt(*args, **kwargs).iloc[0]
+                data = a.to_frame().kurt(*args, **kwargs).iloc[0]
+                if is_cudf(data):  # pragma: no cover
+                    data = data.copy()
+                return data
 
             def _group_kurt(x, *args, **kwargs):
                 if kwargs.get('numeric_only') is not None:
@@ -103,19 +124,18 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
     _agg_funcs = ListField('agg_funcs')
     _post_funcs = ListField('post_funcs')
     _index_levels = Int32Field('index_levels')
-    # for chunk
-    _tileable_op_key = StringField('tileable_op_key')
+    _size_recorder_name = StringField('size_recorder_name')
 
     def __init__(self, raw_func=None, raw_func_kw=None, func=None, func_rename=None,
                  method=None, groupby_params=None, use_inf_as_na=None, combine_size=None,
-                 pre_funcs=None, agg_funcs=None, post_funcs=None, stage=None,
-                 output_types=None, tileable_op_key=None, index_levels=None, **kw):
+                 pre_funcs=None, agg_funcs=None, post_funcs=None, index_levels=None,
+                 size_recorder_name=None, output_types=None, **kw):
         super().__init__(_raw_func=raw_func, _raw_func_kw=raw_func_kw, _func=func,
                          _func_rename=func_rename, _method=method, _groupby_params=groupby_params,
                          _combine_size=combine_size, _use_inf_as_na=use_inf_as_na,
                          _pre_funcs=pre_funcs, _agg_funcs=agg_funcs, _post_funcs=post_funcs,
-                         _stage=stage, _output_types=output_types,
-                         _tileable_op_key=tileable_op_key, _index_levels=index_levels, **kw)
+                         _index_levels=index_levels, _size_recorder_name=size_recorder_name,
+                         _output_types=output_types, **kw)
 
     @property
     def raw_func(self):
@@ -165,17 +185,13 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
     def index_levels(self) -> int:
         return self._index_levels
 
-    @property
-    def tileable_op_key(self):
-        return self._tileable_op_key
-
     def _set_inputs(self, inputs):
         super()._set_inputs(inputs)
         inputs_iter = iter(self._inputs[1:])
         if len(self._inputs) > 1:
             by = []
             for v in self._groupby_params['by']:
-                if isinstance(v, (Base, Entity)):
+                if isinstance(v, ENTITY_TYPE):
                     by.append(next(inputs_iter))
                 else:
                     by.append(v)
@@ -184,7 +200,7 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
     def _get_inputs(self, inputs):
         if isinstance(self._groupby_params['by'], list):
             for v in self._groupby_params['by']:
-                if isinstance(v, (Base, Entity)):
+                if isinstance(v, ENTITY_TYPE):
                     inputs.append(v)
         return inputs
 
@@ -276,8 +292,7 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
         reduce_chunks = []
         for out_idx in itertools.product(*(range(s) for s in chunk_shape)):
             reduce_op = DataFrameGroupByOperand(
-                stage=OperandStage.reduce, shuffle_key=','.join(str(idx) for idx in out_idx),
-                output_types=[OutputType.dataframe_groupby])
+                stage=OperandStage.reduce, output_types=[OutputType.dataframe_groupby])
             reduce_chunks.append(
                 reduce_op.new_chunk([proxy_chunk], shape=(np.nan, np.nan), index=out_idx,
                                     index_value=None))
@@ -285,9 +300,13 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
         return reduce_chunks
 
     @classmethod
-    def _gen_map_chunks(cls, op: "DataFrameGroupByAgg", in_df, out_df, func_infos: ReductionSteps):
+    def _gen_map_chunks(cls,
+                        op: "DataFrameGroupByAgg",
+                        in_chunks: List[ChunkType],
+                        out_df: TileableType,
+                        func_infos: ReductionSteps):
         map_chunks = []
-        for chunk in in_df.chunks:
+        for chunk in in_chunks:
             chunk_inputs = [chunk]
             map_op = op.copy().reset_key()
             # force as_index=True for map phase
@@ -297,14 +316,14 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
             if isinstance(map_op._groupby_params['by'], list):
                 by = []
                 for v in map_op._groupby_params['by']:
-                    if isinstance(v, (Base, Entity)):
+                    if isinstance(v, ENTITY_TYPE):
                         by_chunk = v.cix[chunk.index[0], ]
                         chunk_inputs.append(by_chunk)
                         by.append(by_chunk)
                     else:
                         by.append(v)
                 map_op._groupby_params['by'] = by
-            map_op._stage = OperandStage.map
+            map_op.stage = OperandStage.map
             map_op._pre_funcs = func_infos.pre_funcs
             map_op._agg_funcs = func_infos.agg_funcs
             new_index = chunk.index if len(chunk.index) == 2 else (chunk.index[0], 0)
@@ -341,17 +360,22 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
         return compiler.compile()
 
     @classmethod
-    def _tile_with_shuffle(cls, op: "DataFrameGroupByAgg"):
-        in_df = op.inputs[0]
-        if len(in_df.shape) > 1:
-            in_df = build_concatenated_rows_frame(in_df)
-        out_df = op.outputs[0]
-
-        func_infos = cls._compile_funcs(op, in_df)
-
+    def _tile_with_shuffle(cls,
+                           op: "DataFrameGroupByAgg",
+                           in_df: TileableType,
+                           out_df: TileableType,
+                           func_infos: ReductionSteps):
         # First, perform groupby and aggregation on each chunk.
-        agg_chunks = cls._gen_map_chunks(op, in_df, out_df, func_infos)
+        agg_chunks = cls._gen_map_chunks(op, in_df.chunks, out_df, func_infos)
+        return cls._perform_shuffle(op, agg_chunks, in_df, out_df, func_infos)
 
+    @classmethod
+    def _perform_shuffle(cls,
+                         op: "DataFrameGroupByAgg",
+                         agg_chunks: List[ChunkType],
+                         in_df: TileableType,
+                         out_df: TileableType,
+                         func_infos: ReductionSteps):
         # Shuffle the aggregation chunk.
         reduce_chunks = cls._gen_shuffle_chunks(op, in_df, agg_chunks)
 
@@ -359,13 +383,13 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
         agg_chunks = []
         for chunk in reduce_chunks:
             agg_op = op.copy().reset_key()
-            agg_op._tileable_op_key = op.key
+            agg_op.tileable_op_key = op.key
             agg_op._groupby_params = agg_op.groupby_params.copy()
             agg_op._groupby_params.pop('selection', None)
             # use levels instead of by for reducer
             agg_op._groupby_params.pop('by', None)
             agg_op._groupby_params['level'] = list(range(op.index_levels))
-            agg_op._stage = OperandStage.agg
+            agg_op.stage = OperandStage.agg
             agg_op._agg_funcs = func_infos.agg_funcs
             agg_op._post_funcs = func_infos.post_funcs
             if op.output_types[0] == OutputType.dataframe:
@@ -388,15 +412,21 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
         return new_op.new_tileables([in_df], **kw)
 
     @classmethod
-    def _tile_with_tree(cls, op: "DataFrameGroupByAgg"):
-        in_df = op.inputs[0]
-        if len(in_df.shape) > 1:
-            in_df = build_concatenated_rows_frame(in_df)
-        out_df = op.outputs[0]
+    def _tile_with_tree(cls,
+                        op: "DataFrameGroupByAgg",
+                        in_df: TileableType,
+                        out_df: TileableType,
+                        func_infos: ReductionSteps):
+        chunks = cls._gen_map_chunks(op, in_df.chunks, out_df, func_infos)
+        return cls._combine_tree(op, chunks, out_df, func_infos)
 
-        func_infos = cls._compile_funcs(op, in_df)
+    @classmethod
+    def _combine_tree(cls,
+                      op: "DataFrameGroupByAgg",
+                      chunks: List[ChunkType],
+                      out_df: TileableType,
+                      func_infos: ReductionSteps):
         combine_size = op.combine_size
-        chunks = cls._gen_map_chunks(op, in_df, out_df, func_infos)
         while len(chunks) > combine_size:
             new_chunks = []
             for idx, i in enumerate(range(0, len(chunks), combine_size)):
@@ -410,9 +440,9 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
                         c._index = (j, 0)
                     chk = concat_op.new_chunk(chks, dtypes=chks[0].dtypes)
                 chunk_op = op.copy().reset_key()
-                chunk_op._tileable_op_key = None
+                chunk_op.tileable_op_key = None
                 chunk_op.output_types = [OutputType.dataframe]
-                chunk_op._stage = OperandStage.combine
+                chunk_op.stage = OperandStage.combine
                 chunk_op._groupby_params = chunk_op.groupby_params.copy()
                 chunk_op._groupby_params.pop('selection', None)
                 # use levels instead of by for agg
@@ -430,8 +460,8 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
         concat_op = DataFrameConcat(output_types=[OutputType.dataframe])
         chk = concat_op.new_chunk(chunks, dtypes=chunks[0].dtypes)
         chunk_op = op.copy().reset_key()
-        chunk_op._tileable_op_key = op.key
-        chunk_op._stage = OperandStage.agg
+        chunk_op.tileable_op_key = op.key
+        chunk_op.stage = OperandStage.agg
         chunk_op._groupby_params = chunk_op.groupby_params.copy()
         chunk_op._groupby_params.pop('selection', None)
         # use levels instead of by for agg
@@ -452,18 +482,59 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
         kw.update(dict(chunks=[chunk], nsplits=nsplits))
         return new_op.new_tileables(op.inputs, **kw)
 
+    @classmethod
+    def _tile_auto(cls,
+                  op: "DataFrameGroupByAgg",
+                  in_df: TileableType,
+                  out_df: TileableType,
+                  func_infos: ReductionSteps):
+        ctx = get_context()
+        combine_size = op.combine_size
+        size_recorder_name = str(uuid.uuid4())
+        size_recorder = ctx.create_remote_object(size_recorder_name, SizeRecorder)
+
+        # collect the first combine_size chunks, run it
+        # to get the size before and after agg
+        chunks = in_df.chunks[:combine_size]
+        chunks = cls._gen_map_chunks(op, chunks, out_df, func_infos)
+        for chunk in chunks:
+            chunk.op._size_recorder_name = size_recorder_name
+        # yield to trigger execution
+        yield chunks
+
+        raw_size, agg_size = size_recorder.get()
+        # destroy size recorder
+        ctx.destroy_remote_object(size_recorder_name)
+
+        left_chunks = in_df.chunks[combine_size:]
+        left_chunks = cls._gen_map_chunks(op, left_chunks, out_df, func_infos)
+        if raw_size >= agg_size * len(chunks):
+            # aggregated size is less than 1 chunk
+            # use tree aggregation
+            return cls._combine_tree(op, chunks + left_chunks, out_df, func_infos)
+        else:
+            # otherwise, use shuffle
+            return cls._perform_shuffle(op, chunks + left_chunks,
+                                        in_df, out_df, func_infos)
+
     @classmethod
     def tile(cls, op: "DataFrameGroupByAgg"):
+        in_df = op.inputs[0]
+        if len(in_df.shape) > 1:
+            in_df = build_concatenated_rows_frame(in_df)
+        out_df = op.outputs[0]
+
+        func_infos = cls._compile_funcs(op, in_df)
+
         if op.method == 'auto':
-            ctx = get_context()
-            if ctx is not None and ctx.running_mode == RunningMode.distributed:  # pragma: no cover
-                return cls._tile_with_shuffle(op)
+            if len(in_df.chunks) < op.combine_size:
+                return cls._tile_with_tree(op, in_df, out_df, func_infos)
             else:
-                return cls._tile_with_tree(op)
+                return (yield from cls._tile_auto(op, in_df, out_df, func_infos))
         if op.method == 'shuffle':
-            return cls._tile_with_shuffle(op)
+            return cls._tile_with_shuffle(op, in_df, out_df, func_infos)
         elif op.method == 'tree':
-            return cls._tile_with_tree(op)
+            return cls._tile_with_tree(op, in_df, out_df, func_infos)
         else:  # pragma: no cover
             raise NotImplementedError
 
@@ -482,7 +553,7 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
         elif isinstance(params.get('by'), list):
             new_by = []
             for v in params['by']:
-                if isinstance(v, (Base, Entity)):
+                if isinstance(v, ENTITY_TYPE):
                     new_by.append(ctx[v.key])
                 else:
                     new_by.append(v)
@@ -550,7 +621,12 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
                 else:
                     r.index = xdf.MultiIndex.from_tuples([group_key], names=input_objs[0].grouper.names)
             results.append(result)
-        concat_result = tuple(xdf.concat(parts) for parts in zip(*results))
+        if not results and op.stage == OperandStage.agg:
+            empty_df = pd.DataFrame([], columns=out.dtypes.index,
+                                    index=out.index_value.to_pandas()[:0])
+            concat_result = (empty_df.astype(out.dtypes),)
+        else:
+            concat_result = tuple(xdf.concat(parts) for parts in zip(*results))
         return concat_result
 
     @staticmethod
@@ -629,6 +705,14 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
                 agg_dfs.extend(cls._do_custom_agg(op, custom_reduction, input_obj))
             else:
                 agg_dfs.append(cls._do_predefined_agg(input_obj, map_func_name, **kwds))
+
+        if op._size_recorder_name is not None:
+            # record_size
+            raw_size = len(in_data)
+            agg_size = len(agg_dfs[0])
+            size_recorder = ctx.get_remote_object(op._size_recorder_name)
+            size_recorder.record(raw_size, agg_size)
+
         ctx[op.outputs[0].key] = tuple(agg_dfs)
 
     @classmethod
@@ -721,6 +805,8 @@ class DataFrameGroupByAgg(DataFrameOperand, DataFrameOperandMixin):
             result = xdf.concat(aggs)
             if result.ndim == 2:
                 result = result.iloc[:, 0]
+                if is_cudf(result):  # pragma: no cover
+                    result = result.copy()
             result.name = out_chunk.name
 
         ctx[out_chunk.key] = result
@@ -769,6 +855,8 @@ def agg(groupby, func=None, method='auto', *args, **kwargs):
     if not isinstance(groupby, GROUPBY_TYPE):
         raise TypeError(f'Input should be type of groupby, not {type(groupby)}')
 
+    if method is None:
+        method = 'auto'
     if method not in ['shuffle', 'tree', 'auto']:
         raise ValueError(f"Method {method} is not available, "
                          "please specify 'tree' or 'shuffle")
