@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,20 +19,23 @@ import numpy as np
 import pandas as pd
 
 from ... import opcodes as OperandDef
-from ...core import Entity, Base, OutputType
+from ...core import ENTITY_TYPE, Entity, OutputType
+from ...core.operand import OperandStage, MapReduceOperand
 from ...lib.groupby_wrapper import wrapped_groupby
-from ...operands import OperandStage
-from ...serialize import BoolField, Int32Field, AnyField
-from ...utils import get_shuffle_input_keys_idxes
+from ...serialization.serializables import BoolField, Int32Field, AnyField
+from ...utils import lazy_import
 from ..align import align_dataframe_series, align_series_series
 from ..initializer import Series as asseries
 from ..core import SERIES_TYPE, SERIES_CHUNK_TYPE
 from ..utils import build_concatenated_rows_frame, hash_dataframe_on, \
-    build_df, build_series, parse_index
-from ..operands import DataFrameOperandMixin, DataFrameMapReduceOperand, DataFrameShuffleProxy
+    build_df, build_series, parse_index, is_cudf
+from ..operands import DataFrameOperandMixin, DataFrameShuffleProxy
 
 
-class DataFrameGroupByOperand(DataFrameMapReduceOperand, DataFrameOperandMixin):
+cudf = lazy_import('cudf', globals=globals())
+
+
+class DataFrameGroupByOperand(MapReduceOperand, DataFrameOperandMixin):
     _op_type_ = OperandDef.GROUPBY
 
     _by = AnyField('by', on_serialize=lambda x: x.data if isinstance(x, Entity) else x)
@@ -44,12 +47,12 @@ class DataFrameGroupByOperand(DataFrameMapReduceOperand, DataFrameOperandMixin):
     _shuffle_size = Int32Field('shuffle_size')
 
     def __init__(self, by=None, level=None, as_index=None, sort=None, group_keys=None,
-                 shuffle_size=None, stage=None, shuffle_key=None, output_types=None, **kw):
+                 shuffle_size=None, output_types=None, **kw):
         super().__init__(_by=by, _level=level, _as_index=as_index, _sort=sort,
-                         _group_keys=group_keys, _shuffle_size=shuffle_size, _stage=stage,
-                         _shuffle_key=shuffle_key, _output_types=output_types, **kw)
+                         _group_keys=group_keys, _shuffle_size=shuffle_size,
+                         _output_types=output_types, **kw)
         if output_types:
-            if stage in (OperandStage.map, OperandStage.reduce):
+            if self.stage in (OperandStage.map, OperandStage.reduce):
                 if output_types[0] in (OutputType.dataframe, OutputType.dataframe_groupby):
                     output_types = [OutputType.dataframe]
                 else:
@@ -108,7 +111,7 @@ class DataFrameGroupByOperand(DataFrameMapReduceOperand, DataFrameOperandMixin):
         if isinstance(new_kw['by'], list):
             new_by = []
             for v in new_kw['by']:
-                if isinstance(v, (Base, Entity)):
+                if isinstance(v, ENTITY_TYPE):
                     build_fun = build_df if v.ndim == 2 else build_series
                     mock_by = pd.concat([
                         build_fun(v, size=2, fill_value=1, name=v.name),
@@ -214,7 +217,7 @@ class DataFrameGroupByOperand(DataFrameMapReduceOperand, DataFrameOperandMixin):
         map_chunks = []
         for chunk in in_df.chunks:
             map_op = op.copy().reset_key()
-            map_op._stage = OperandStage.map
+            map_op.stage = OperandStage.map
             map_op._shuffle_size = chunk_shape[0]
             map_op._output_types = [output_type]
             chunk_inputs = [chunk]
@@ -235,12 +238,10 @@ class DataFrameGroupByOperand(DataFrameMapReduceOperand, DataFrameOperandMixin):
         # generate reduce chunks
         reduce_chunks = []
         for out_idx in itertools.product(*(range(s) for s in chunk_shape)):
-            shuffle_key = ','.join(str(idx) for idx in out_idx)
             reduce_op = op.copy().reset_key()
             reduce_op._by = None
             reduce_op._output_types = [output_type]
-            reduce_op._stage = OperandStage.reduce
-            reduce_op._shuffle_key = shuffle_key
+            reduce_op.stage = OperandStage.reduce
             reduce_chunks.append(
                 reduce_op.new_chunk([proxy_chunk], shape=(np.nan, np.nan), index=out_idx))
 
@@ -284,7 +285,7 @@ class DataFrameGroupByOperand(DataFrameMapReduceOperand, DataFrameOperandMixin):
         if isinstance(by, list):
             new_by = []
             for v in by:
-                if isinstance(v, (Base, Entity)):
+                if isinstance(v, ENTITY_TYPE):
                     deliver_by = True
                     new_by.append(ctx[v.key])
                 else:
@@ -305,13 +306,15 @@ class DataFrameGroupByOperand(DataFrameMapReduceOperand, DataFrameOperandMixin):
             result = src.iloc[f]
             if src.index.names:
                 result.index.names = src.index.names
+            if is_cudf(result):  # pragma: no cover
+                result = result.copy()
             return result
 
         for index_idx, index_filter in enumerate(filters):
             if is_dataframe_obj:
-                group_key = ','.join([str(index_idx), str(chunk.index[1])])
+                reducer_index = (index_idx, chunk.index[1])
             else:
-                group_key = str(index_idx)
+                reducer_index = (index_idx,)
 
             if deliver_by:
                 filtered_by = []
@@ -321,31 +324,26 @@ class DataFrameGroupByOperand(DataFrameMapReduceOperand, DataFrameOperandMixin):
                     else:
                         filtered_by.append(v)
                 if isinstance(df, tuple):
-                    ctx[(chunk.key, group_key)] = tuple(_take_index(x, index_filter) for x in df) \
+                    ctx[chunk.key, reducer_index] = tuple(_take_index(x, index_filter) for x in df) \
                         + (filtered_by, deliver_by)
                 else:
-                    ctx[(chunk.key, group_key)] = (_take_index(df, index_filter), filtered_by, deliver_by)
+                    ctx[chunk.key, reducer_index] = (_take_index(df, index_filter), filtered_by, deliver_by)
             else:
                 if isinstance(df, tuple):
-                    ctx[(chunk.key, group_key)] = tuple(_take_index(x, index_filter) for x in df) + (deliver_by,)
+                    ctx[chunk.key, reducer_index] = tuple(_take_index(x, index_filter) for x in df) + (deliver_by,)
                 else:
-                    ctx[(chunk.key, group_key)] = _take_index(df, index_filter)
+                    ctx[chunk.key, reducer_index] = _take_index(df, index_filter)
 
     @classmethod
-    def execute_reduce(cls, ctx, op):
-        is_dataframe_obj = op.inputs[0].op.output_types[0] == OutputType.dataframe
+    def execute_reduce(cls, ctx, op: "DataFrameGroupByOperand"):
+        xdf = cudf if op.gpu else pd
         chunk = op.outputs[0]
-        input_keys, input_idxes = get_shuffle_input_keys_idxes(op.inputs[0])
-        input_idx_to_df = {idx: ctx[inp_key, ','.join(str(ix) for ix in chunk.index)]
-                           for inp_key, idx in zip(input_keys, input_idxes)}
-        row_idxes = sorted({idx[0] for idx in input_idx_to_df})
+        input_idx_to_df = dict(op.iter_mapper_data_with_index(ctx))
+        row_idxes = sorted(input_idx_to_df.keys())
 
         res = []
         for row_idx in row_idxes:
-            if is_dataframe_obj:
-                row_df = input_idx_to_df.get((row_idx, 0), None)
-            else:
-                row_df = input_idx_to_df.get((row_idx,), None)
+            row_df = input_idx_to_df.get(row_idx, None)
             if row_df is not None:
                 res.append(row_df)
         by = None
@@ -356,7 +354,7 @@ class DataFrameGroupByOperand(DataFrameMapReduceOperand, DataFrameOperandMixin):
             part_len = len(res[0])
             part_len -= 1 if not deliver_by else 2
             for n in range(part_len):
-                r.append(pd.concat([it[n] for it in res], axis=0))
+                r.append(xdf.concat([it[n] for it in res], axis=0))
             r = tuple(r)
 
             if deliver_by:
