@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,15 +21,15 @@ from pandas.core.dtypes.cast import find_common_type
 from pandas.core.indexing import IndexingError
 
 from ... import opcodes as OperandDef
-from ...core import Entity, Base, OutputType
+from ...core import ENTITY_TYPE, OutputType, recursive_tile
 from ...config import options
-from ...serialize import AnyField, KeyField, ListField
+from ...serialization.serializables import AnyField, KeyField, ListField
 from ...tensor import asarray
 from ...tensor.datasource.empty import empty
 from ...tensor.indexing.core import calc_shape
 from ...utils import ceildiv
 from ..operands import DataFrameOperand, DataFrameOperandMixin, DATAFRAME_TYPE
-from ..utils import indexing_index_value
+from ..utils import indexing_index_value, is_cudf
 from .index_lib import DataFrameIlocIndexesHandler
 
 
@@ -68,8 +68,8 @@ def process_iloc_indexes(inp, indexes):
                             raise TypeError(f'cannot do slice indexing on {type(pd_index)} '
                                             f'with these indexers [{val}] of {type(val)}')
             new_indexes.append(index)
-        elif isinstance(index, (list, np.ndarray, pd.Series, Base, Entity)):
-            if not isinstance(index, (Base, Entity)):
+        elif isinstance(index, (list, np.ndarray, pd.Series, ENTITY_TYPE)):
+            if not isinstance(index, ENTITY_TYPE):
                 index = np.asarray(index)
             else:
                 index = asarray(index)
@@ -144,8 +144,11 @@ class HeadTailOptimizedOperandMixin(DataFrameOperandMixin):
             return False
         if index0.step is not None and index0.step != 1:
             return False
-        if len(indexes) == 2 and indexes[1] != slice(None):
-            return False
+        if len(indexes) == 2:
+            if not isinstance(indexes[1], slice):
+                return False
+            if indexes[1] != slice(None):
+                return False
         if cls._is_tail(index0):
             # tail
             return True
@@ -250,11 +253,11 @@ class DataFrameIlocGetItem(DataFrameOperand, HeadTailOptimizedOperandMixin):
         self._input = next(inputs_iter)
         indexes = []
         for index in self._indexes:
-            if isinstance(index, (Entity, Base)):
+            if isinstance(index, ENTITY_TYPE):
                 indexes.append(next(inputs_iter))
             else:
                 indexes.append(index)
-        self._indexes = tuple(indexes)
+        self._indexes = indexes
 
     def __call__(self, df):
         # Note [Fancy Index of Numpy and Pandas]
@@ -273,7 +276,7 @@ class DataFrameIlocGetItem(DataFrameOperand, HeadTailOptimizedOperandMixin):
         shape0 = tuple(calc_shape((df.shape[0],), (self.indexes[0],)))
         shape1 = tuple(calc_shape((df.shape[1],), (self.indexes[1],)))
 
-        inputs = [df] + [index for index in self._indexes if isinstance(index, (Base, Entity))]
+        inputs = [df] + [index for index in self._indexes if isinstance(index, ENTITY_TYPE)]
 
         # NB: pandas only compresses the result to series when index on one of axis is integral
         if isinstance(self.indexes[1], Integral):
@@ -341,7 +344,7 @@ class DataFrameIlocGetItem(DataFrameOperand, HeadTailOptimizedOperandMixin):
             return tileds
 
         handler = DataFrameIlocIndexesHandler()
-        return [handler.handle(op)]
+        return [(yield from handler.handle(op))]
 
     @classmethod
     def execute(cls, ctx, op):
@@ -355,6 +358,8 @@ class DataFrameIlocGetItem(DataFrameOperand, HeadTailOptimizedOperandMixin):
         r = df.iloc[indexes]
         if isinstance(r, pd.Series) and r.dtype != chunk.dtype:
             r = r.astype(chunk.dtype)
+        if is_cudf(r):  # pragma: no cover
+            r = r.copy()
         ctx[chunk.key] = r
 
 
@@ -389,8 +394,10 @@ class DataFrameIlocSetItem(DataFrameOperand, DataFrameOperandMixin):
         out_df = op.outputs[0]
 
         # See Note [Fancy Index of Numpy and Pandas]
-        tensor0 = empty(in_df.shape[0], chunk_size=(in_df.nsplits[0],))[op.indexes[0]].tiles()
-        tensor1 = empty(in_df.shape[1], chunk_size=(in_df.nsplits[1],))[op.indexes[1]].tiles()
+        tensor0 = yield from recursive_tile(
+            empty(in_df.shape[0], chunk_size=(in_df.nsplits[0],))[op.indexes[0]])
+        tensor1 = yield from recursive_tile(
+            empty(in_df.shape[1], chunk_size=(in_df.nsplits[1],))[op.indexes[1]])
 
         chunk_mapping = {c0.inputs[0].index + c1.inputs[0].index: (c0, c1)
                          for c0, c1 in itertools.product(tensor0.chunks, tensor1.chunks)}
@@ -402,7 +409,7 @@ class DataFrameIlocSetItem(DataFrameOperand, DataFrameOperandMixin):
             else:
                 chunk_op = op.copy().reset_key()
                 index_chunk, column_chunk = chunk_mapping[chunk.index]
-                chunk_op._indexes = (index_chunk.op.indexes[0], column_chunk.op.indexes[0])
+                chunk_op._indexes = [index_chunk.op.indexes[0], column_chunk.op.indexes[0]]
                 chunk_op._value = op.value
                 out_chunk = chunk_op.new_chunk([chunk],
                                                shape=chunk.shape, index=chunk.index, dtypes=chunk.dtypes,
@@ -450,11 +457,11 @@ class SeriesIlocGetItem(DataFrameOperand, HeadTailOptimizedOperandMixin):
 
         indexes = []
         for index in self._indexes:
-            if isinstance(index, (Entity, Base)):
+            if isinstance(index, ENTITY_TYPE):
                 indexes.append(next(inputs_iter))
             else:
                 indexes.append(index)
-        self._indexes = tuple(indexes)
+        self._indexes = indexes
 
     @classmethod
     def tile(cls, op):
@@ -463,7 +470,7 @@ class SeriesIlocGetItem(DataFrameOperand, HeadTailOptimizedOperandMixin):
             return tileds
 
         handler = DataFrameIlocIndexesHandler()
-        return [handler.handle(op)]
+        return [(yield from handler.handle(op))]
 
     @classmethod
     def execute(cls, ctx, op):
@@ -486,7 +493,7 @@ class SeriesIlocGetItem(DataFrameOperand, HeadTailOptimizedOperandMixin):
         else:
             shape = tuple(calc_shape(series.shape, self.indexes))
             index_value = indexing_index_value(series.index_value, self.indexes[0])
-            inputs = [series] + [index for index in self._indexes if isinstance(index, (Base, Entity))]
+            inputs = [series] + [index for index in self._indexes if isinstance(index, ENTITY_TYPE)]
             return self.new_series(inputs, shape=shape, dtype=series.dtype,
                                    index_value=index_value, name=series.name)
 
@@ -520,7 +527,8 @@ class SeriesIlocSetItem(DataFrameOperand, DataFrameOperandMixin):
         out = op.outputs[0]
 
         # Reuse the logic of fancy indexing in tensor module.
-        tensor = empty(in_series.shape, chunk_size=in_series.nsplits)[op.indexes[0]].tiles()
+        tensor = yield from recursive_tile(
+            empty(in_series.shape, chunk_size=in_series.nsplits)[op.indexes[0]])
 
         chunk_mapping = dict((c.inputs[0].index, c) for c in tensor.chunks)
 
@@ -578,16 +586,16 @@ class IndexIlocGetItem(DataFrameOperand, DataFrameOperandMixin):
 
         indexes = []
         for index in self._indexes:
-            if isinstance(index, (Entity, Base)):
+            if isinstance(index, ENTITY_TYPE):
                 indexes.append(next(inputs_iter))
             else:
                 indexes.append(index)
-        self._indexes = tuple(indexes)
+        self._indexes = indexes
 
     @classmethod
     def tile(cls, op):
         handler = DataFrameIlocIndexesHandler()
-        return [handler.handle(op)]
+        return [(yield from handler.handle(op))]
 
     @classmethod
     def execute(cls, ctx, op):
@@ -606,7 +614,7 @@ class IndexIlocGetItem(DataFrameOperand, DataFrameOperandMixin):
         else:
             shape = tuple(calc_shape(idx.shape, self.indexes))
             index_value = indexing_index_value(idx.index_value, self.indexes[0])
-            inputs = [idx] + [index for index in self._indexes if isinstance(index, (Base, Entity))]
+            inputs = [idx] + [index for index in self._indexes if isinstance(index, ENTITY_TYPE)]
             return self.new_index(inputs, shape=shape, dtype=idx.dtype,
                                   index_value=index_value, name=idx.name)
 
