@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,9 +21,9 @@ from pandas.core.dtypes.cast import find_common_type
 from pandas.core.indexing import IndexingError
 
 from ... import opcodes as OperandDef
-from ...core import Base, Entity
-from ...operands import OperandStage
-from ...serialize import KeyField, ListField
+from ...core import ENTITY_TYPE
+from ...core.operand import OperandStage
+from ...serialization.serializables import KeyField, ListField
 from ...tensor.datasource import asarray
 from ...tensor.utils import calc_sliced_size, filter_inputs
 from ...utils import lazy_import
@@ -48,8 +48,8 @@ def process_loc_indexes(inp, indexes):
 
     new_indexes = []
     for ax, index in enumerate(indexes):
-        if isinstance(index, (list, np.ndarray, pd.Series, Base, Entity)):
-            if not isinstance(index, (Base, Entity)):
+        if isinstance(index, (list, np.ndarray, pd.Series, ENTITY_TYPE)):
+            if not isinstance(index, ENTITY_TYPE):
                 index = np.asarray(index)
             else:
                 index = asarray(index)
@@ -136,11 +136,11 @@ class DataFrameLocGetItem(DataFrameOperand, DataFrameOperandMixin):
         self._input = next(inputs_iter)
         indexes = []
         for index in self._indexes:
-            if isinstance(index, (Entity, Base)):
+            if isinstance(index, ENTITY_TYPE):
                 indexes.append(next(inputs_iter))
             else:
                 indexes.append(index)
-        self._indexes = tuple(indexes)
+        self._indexes = list(indexes)
 
     @classmethod
     def _calc_slice_param(cls,
@@ -330,7 +330,7 @@ class DataFrameLocGetItem(DataFrameOperand, DataFrameOperandMixin):
     @classmethod
     def tile(cls, op):
         handler = DataFrameLocIndexesHandler()
-        return [handler.handle(op)]
+        return [(yield from handler.handle(op))]
 
     @classmethod
     def execute(cls, ctx, op):
