@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,9 +17,9 @@ from functools import reduce
 
 import pandas as pd
 
-from ..core import FuseChunkData, FuseChunk, Base, Entity, OutputType
-from ..operands import Operand, TileableOperandMixin, MapReduceOperand, Fuse
-from ..operands import ShuffleProxy, FuseChunkMixin
+from ..core import FuseChunkData, FuseChunk, ENTITY_TYPE, OutputType
+from ..core.operand import Operand, TileableOperandMixin, Fuse, \
+    ShuffleProxy, FuseChunkMixin
 from ..tensor.core import TENSOR_TYPE
 from ..tensor.operands import TensorOperandMixin
 from ..utils import calc_nsplits
@@ -107,7 +107,7 @@ class DataFrameOperandMixin(TileableOperandMixin):
         if isinstance(groupby_params['by'], list):
             by = []
             for v in groupby_params['by']:
-                if isinstance(v, (Base, Entity)):
+                if isinstance(v, ENTITY_TYPE):
                     by.append(cls.concat_tileable_chunks(v).chunks[0])
                 else:
                     by.append(v)
@@ -120,7 +120,7 @@ class DataFrameOperandMixin(TileableOperandMixin):
         chunk_inputs = list(groupby.chunks)
         if isinstance(groupby_params['by'], list):
             for chunk_v, v in zip(groupby_params['by'], groupby.op.groupby_params['by']):
-                if isinstance(v, (Base, Entity)):
+                if isinstance(v, ENTITY_TYPE):
                     inputs.append(v)
                     chunk_inputs.append(chunk_v)
         return inputs, chunk_inputs
@@ -263,9 +263,6 @@ class DataFrameShuffleProxy(ShuffleProxy, DataFrameOperandMixin):
         pass
 
 
-DataFrameMapReduceOperand = MapReduceOperand
-
-
 class DataFrameFuseChunkMixin(FuseChunkMixin, DataFrameOperandMixin):
     __slots__ = ()
 
