@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,6 @@ import functools
 import itertools
 from collections import OrderedDict
 from collections.abc import Iterable
-from distutils.version import LooseVersion
 from typing import List, Dict, Union
 
 import numpy as np
@@ -25,11 +24,12 @@ import pandas as pd
 
 from ... import opcodes, tensor as mars_tensor
 from ...config import options
-from ...core import OutputType, Base, Entity
-from ...custom_log import redirect_custom_log
-from ...operands import OperandStage
-from ...serialize import BoolField, AnyField, Int32Field, ListField, DictField
-from ...utils import ceildiv, lazy_import, enter_mode, enter_current_session
+from ...core import OutputType, ENTITY_TYPE, enter_mode, recursive_tile
+from ...core.custom_log import redirect_custom_log
+from ...core.operand import OperandStage
+from ...lib.version import parse as parse_version
+from ...serialization.serializables import BoolField, AnyField, Int32Field, ListField, DictField
+from ...utils import ceildiv, lazy_import, enter_current_session
 from ..core import INDEX_CHUNK_TYPE
 from ..merge import DataFrameConcat
 from ..operands import DataFrameOperand, DataFrameOperandMixin
@@ -40,17 +40,16 @@ from .core import CustomReduction, ReductionCompiler, ReductionSteps, ReductionP
 cp = lazy_import('cupy', globals=globals(), rename='cp')
 cudf = lazy_import('cudf', globals=globals())
 
-_agg_size_as_series = LooseVersion(pd.__version__) >= '1.3.0'
+_agg_size_as_series = parse_version(pd.__version__) >= parse_version('1.3.0')
 
 
 def where_function(cond, var1, var2):
-    if var1.ndim >= 1:
+    if hasattr(var1, 'ndim') and var1.ndim >= 1:
         return var1.where(cond, var2)
+    elif isinstance(var1, ENTITY_TYPE):
+        return mars_tensor.where(cond, var1, var2)
     else:
-        if isinstance(var1, (Base, Entity)):
-            return mars_tensor.where(cond, var1, var2)
-        else:
-            return np.where(cond, var1, var2).item()
+        return np.where(cond, var1, var2).item()
 
 
 _agg_functions = {
@@ -98,7 +97,7 @@ class DataFrameAggregate(DataFrameOperand, DataFrameOperandMixin):
                          _func_rename=func_rename, _axis=axis, _use_inf_as_na=use_inf_as_na,
                          _numeric_only=numeric_only, _bool_only=bool_only,
                          _combine_size=combine_size, _pre_funcs=pre_funcs, _agg_funcs=agg_funcs,
-                         _post_funcs=post_funcs, _stage=stage, _output_types=output_types, **kw)
+                         _post_funcs=post_funcs, _output_types=output_types, stage=stage, **kw)
 
     @property
     def raw_func(self):
@@ -242,7 +241,7 @@ class DataFrameAggregate(DataFrameOperand, DataFrameOperandMixin):
             func_info = func_infos[new_axis_index]
             # force as_index=True for map phase
             map_op.output_types = [OutputType.dataframe] if chunk.ndim == 2 else [OutputType.series]
-            map_op._stage = OperandStage.map
+            map_op.stage = OperandStage.map
             map_op._pre_funcs = func_info.pre_funcs
             map_op._agg_funcs = func_info.agg_funcs
 
@@ -319,7 +318,8 @@ class DataFrameAggregate(DataFrameOperand, DataFrameOperandMixin):
         nsplits = tuple((1,) * s for s in in_df.chunk_shape)
         tileable = tileable_op.new_tileable(out_df.inputs, chunks=chunks, nsplits=nsplits,
                                             shape=in_df.chunk_shape, dtype=out_df.dtype)
-        return [tileable.sum()._inplace_tile()]
+        ret = yield from recursive_tile(tileable.sum())
+        return [ret]
 
     @staticmethod
     def _add_functions(op: "DataFrameAggregate", compiler: ReductionCompiler,
@@ -414,7 +414,7 @@ class DataFrameAggregate(DataFrameOperand, DataFrameOperandMixin):
                                                   shape=concat_shape, index_value=chks[0].index_value)
                     chunk_op = op.copy().reset_key()
                     chunk_op.output_types = [OutputType.dataframe]
-                    chunk_op._stage = OperandStage.combine
+                    chunk_op.stage = OperandStage.combine
                     chunk_op._agg_funcs = func_info.agg_funcs
 
                     if axis == 0:
@@ -445,7 +445,7 @@ class DataFrameAggregate(DataFrameOperand, DataFrameOperandMixin):
             chk = concat_op.new_chunk(chks, dtypes=dtypes_list[idx] if dtypes_list else None,
                                       shape=concat_shape, index_value=chks[0].index_value)
             chunk_op = op.copy().reset_key()
-            chunk_op._stage = OperandStage.agg
+            chunk_op.stage = OperandStage.agg
             chunk_op._agg_funcs = func_info.agg_funcs
             chunk_op._post_funcs = func_info.post_funcs
 
@@ -514,7 +514,7 @@ class DataFrameAggregate(DataFrameOperand, DataFrameOperandMixin):
         if len(in_df.chunks) == 1:
             return cls._tile_single_chunk(op)
         elif not _agg_size_as_series and in_df.ndim == 2 and op.raw_func == 'size':
-            return cls._tile_size(op)
+            return (yield from cls._tile_size(op))
         else:
             return cls._tile_tree(op)
 
