@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,7 +21,7 @@ except ImportError:  # pragma: no cover
 from ... import opcodes as OperandDef
 from ...core import OutputType
 from ...config import options
-from ...serialize import BoolField
+from ...serialization.serializables import BoolField
 from ...utils import lazy_import
 from ..arrays import ArrowListArray, ArrowListDtype
 from .core import DataFrameReductionOperand, DataFrameReductionMixin, \
@@ -130,7 +130,7 @@ class DataFrameNunique(DataFrameReductionOperand, DataFrameReductionMixin):
         return self._use_arrow_dtype
 
     @classmethod
-    def _make_agg_object(cls, op):
+    def get_reduction_callable(cls, op):
         return NuniqueReduction(name=cls._func_name, axis=op.axis, dropna=op.dropna,
                                 use_arrow_dtype=op.use_arrow_dtype, is_gpu=op.is_gpu())
 
@@ -182,7 +182,7 @@ def nunique_dataframe(df, axis=0, dropna=True, combine_size=None):
     return op(df)
 
 
-def nunique_series(df, dropna=True, combine_size=None):
+def nunique_series(series, dropna=True, combine_size=None):
     """
     Return number of unique elements in the object.
 
@@ -222,4 +222,4 @@ def nunique_series(df, dropna=True, combine_size=None):
     op = DataFrameNunique(dropna=dropna, combine_size=combine_size,
                           output_types=[OutputType.scalar],
                           use_arrow_dtype=options.dataframe.use_arrow_dtype)
-    return op(df)
+    return op(series)
