@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,8 +16,8 @@ import numpy as np
 
 from ... import opcodes
 from ...config import options
-from ...core import OutputType
-from ...serialize import BoolField
+from ...core import OutputType, ENTITY_TYPE
+from ...serialization.serializables import BoolField
 from .core import DataFrameReductionOperand, DataFrameReductionMixin
 
 
@@ -46,25 +46,31 @@ class DataFrameSkew(DataFrameReductionOperand, DataFrameReductionMixin):
                 - 3 * (x ** 2).mean(skipna=skipna) * mean \
                 + 2 * mean ** 3
             var = x.var(skipna=skipna, ddof=0)
-            val = where_function(var > 0, divided / var ** 1.5, np.nan)
+            if isinstance(var, ENTITY_TYPE) or var > 0:
+                val = where_function(var > 0, divided / var ** 1.5, np.nan)
+            else:
+                val = np.nan
             if not bias:
-                val = where_function((var > 0) & (cnt > 2), val * ((cnt * (cnt - 1)) ** 0.5 / (cnt - 2)), np.nan)
+                val = where_function((var > 0) & (cnt > 2),
+                                     val * ((cnt * (cnt - 1)) ** 0.5 / (cnt - 2)),
+                                     np.nan)
             return val
 
         return skew
 
 
-def skew_series(df, axis=None, skipna=None, level=None, combine_size=None, bias=False):
+def skew_series(df, axis=None, skipna=None, level=None, combine_size=None, bias=False, method=None):
     use_inf_as_na = options.dataframe.mode.use_inf_as_na
     op = DataFrameSkew(axis=axis, skipna=skipna, level=level, combine_size=combine_size,
-                       bias=bias, output_types=[OutputType.scalar], use_inf_as_na=use_inf_as_na)
+                       bias=bias, output_types=[OutputType.scalar], use_inf_as_na=use_inf_as_na,
+                       method=method)
     return op(df)
 
 
 def skew_dataframe(df, axis=None, skipna=None, level=None, numeric_only=None, combine_size=None,
-                   bias=False):
+                   bias=False, method=None):
     use_inf_as_na = options.dataframe.mode.use_inf_as_na
     op = DataFrameSkew(axis=axis, skipna=skipna, level=level, numeric_only=numeric_only,
                        bias=bias, combine_size=combine_size, output_types=[OutputType.series],
-                       use_inf_as_na=use_inf_as_na)
+                       use_inf_as_na=use_inf_as_na, method=method)
     return op(df)
