@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,7 @@
 
 from ... import opcodes as OperandDef
 from ...core import OutputType
-from ...serialize import StringField
+from ...serialization.serializables import StringField
 from .core import DataFrameReductionOperand, DataFrameReductionMixin
 
 
@@ -39,8 +39,12 @@ class DataFrameStrConcat(DataFrameReductionOperand, DataFrameReductionMixin):
     def get_reduction_args(self, axis=None):
         return dict(sep=self._sep, na_rep=self._na_rep)
 
+    @property
+    def is_atomic(self):
+        return True
+
     @classmethod
-    def _make_agg_object(cls, op):
+    def get_reduction_callable(cls, op):
         sep, na_rep = op.sep, op.na_rep
 
         def str_concat(obj):
