@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,12 +12,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import functools
-
 from ... import opcodes as OperandDef
 from ...config import options
 from ...core import OutputType
-from ...serialize import Int32Field
+from ...serialization.serializables import Int32Field
 from .core import DataFrameReductionOperand, DataFrameReductionMixin
 
 
@@ -35,24 +33,32 @@ class DataFrameVar(DataFrameReductionOperand, DataFrameReductionMixin):
         return self._ddof
 
     @classmethod
-    def _make_agg_object(cls, op):
-        from .aggregation import var_function
-        pf = functools.partial(var_function, ddof=op.ddof, skipna=op.skipna)
-        pf.__name__ = cls._func_name
-        return pf
+    def get_reduction_callable(cls, op):
+        skipna, ddof = op.skipna, op.ddof
+
+        def var(x):
+            cnt = x.count()
+            if ddof == 0:
+                return (x ** 2).mean(skipna=skipna) - (x.mean(skipna=skipna)) ** 2
+            return ((x ** 2).sum(skipna=skipna) - x.sum(skipna=skipna) ** 2 / cnt) / (cnt - ddof)
+
+        return var
 
 
-def var_series(series, axis=None, skipna=None, level=None, ddof=1, combine_size=None):
+def var_series(series, axis=None, skipna=None, level=None, ddof=1,
+               combine_size=None, method=None):
     use_inf_as_na = options.dataframe.mode.use_inf_as_na
     op = DataFrameVar(axis=axis, skipna=skipna, level=level, ddof=ddof,
                       combine_size=combine_size, output_types=[OutputType.scalar],
-                      use_inf_as_na=use_inf_as_na)
+                      use_inf_as_na=use_inf_as_na, method=method)
     return op(series)
 
 
-def var_dataframe(df, axis=None, skipna=None, level=None, ddof=1, numeric_only=None, combine_size=None):
+def var_dataframe(df, axis=None, skipna=None, level=None, ddof=1, numeric_only=None,
+                  combine_size=None, method=None):
     use_inf_as_na = options.dataframe.mode.use_inf_as_na
     op = DataFrameVar(axis=axis, skipna=skipna, level=level, ddof=ddof,
                       numeric_only=numeric_only, combine_size=combine_size,
-                      output_types=[OutputType.series], use_inf_as_na=use_inf_as_na)
+                      output_types=[OutputType.series], use_inf_as_na=use_inf_as_na,
+                      method=method)
     return op(df)
