@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,14 +18,14 @@ import numpy as np
 import pandas as pd
 
 from ... import opcodes as OperandDef
-from ...context import RunningMode
-from ...utils import lazy_import, get_shuffle_input_keys_idxes
-from ...operands import OperandStage
-from ...serialize import Int32Field, ListField, StringField, BoolField
+from ...core.operand import OperandStage, MapReduceOperand
+from ...utils import lazy_import
+from ...serialization.serializables import Int32Field, ListField, StringField, BoolField
 from ...tensor.base.psrs import PSRSOperandMixin
-from ..utils import standardize_range_index
-from ..operands import DataFrameOperandMixin, DataFrameOperand, DataFrameShuffleProxy, \
-    DataFrameMapReduceOperand
+from ..core import IndexValue, OutputType
+from ..utils import standardize_range_index, parse_index, is_cudf
+from ..operands import DataFrameOperandMixin, DataFrameOperand, \
+    DataFrameShuffleProxy
 
 
 cudf = lazy_import('cudf', globals=globals())
@@ -101,10 +101,16 @@ class DataFramePSRSOperandMixin(DataFrameOperandMixin, PSRSOperandMixin):
 
     @classmethod
     def concat_and_pivot(cls, op, axis_chunk_shape, out_idx, sorted_chunks, sampled_chunks):
+        from .sort_values import DataFrameSortValues
+
         # stage 2: gather and merge samples, choose and broadcast p-1 pivots
         kind = None if op.psrs_kinds is None else op.psrs_kinds[1]
+        if isinstance(op, DataFrameSortValues):
+            output_types = op.output_types
+        else:
+            output_types = [OutputType.index]
         concat_pivot_op = DataFramePSRSConcatPivot(kind=kind, n_partition=axis_chunk_shape,
-                                                   output_types=op.output_types,
+                                                   output_types=output_types,
                                                    **cls._collect_op_properties(op))
         concat_pivot_shape = \
             sorted_chunks[0].shape[:op.axis] + (axis_chunk_shape - 1,) + \
@@ -113,7 +119,7 @@ class DataFramePSRSOperandMixin(DataFrameOperandMixin, PSRSOperandMixin):
         concat_pivot_chunk = concat_pivot_op.new_chunk(sampled_chunks,
                                                        shape=concat_pivot_shape,
                                                        index=concat_pivot_index,
-                                                       output_type=op.output_types[0])
+                                                       output_type=output_types[0])
         return concat_pivot_chunk
 
     @classmethod
@@ -128,9 +134,13 @@ class DataFramePSRSOperandMixin(DataFrameOperandMixin, PSRSOperandMixin):
                                                          stage=OperandStage.map,
                                                          output_types=op.output_types,
                                                          **cls._collect_op_properties(op))
+            if isinstance(chunk_inputs[0].index_value.value, IndexValue.RangeIndex):
+                index_value = parse_index(pd.Int64Index([]))
+            else:
+                index_value = chunk_inputs[0].index_value
             kw = dict(shape=chunk_inputs[0].shape,
                       index=chunk_inputs[0].index,
-                      index_value=chunk_inputs[0].index_value)
+                      index_value=index_value)
             if op.outputs[0].ndim == 2:
                 kw.update(dict(columns_value=chunk_inputs[0].columns_value,
                                dtypes=chunk_inputs[0].dtypes))
@@ -147,7 +157,7 @@ class DataFramePSRSOperandMixin(DataFrameOperandMixin, PSRSOperandMixin):
         for i, partition_chunk in enumerate(partition_chunks):
             kind = None if op.psrs_kinds is None else op.psrs_kinds[2]
             partition_shuffle_reduce = DataFramePSRSShuffle(
-                stage=OperandStage.reduce, kind=kind, shuffle_key=str(i),
+                stage=OperandStage.reduce, kind=kind, reducer_index=(i,),
                 output_types=op.output_types, **cls._collect_op_properties(op))
             chunk_shape = list(partition_chunk.shape)
             chunk_shape[op.axis] = np.nan
@@ -167,7 +177,7 @@ class DataFramePSRSOperandMixin(DataFrameOperandMixin, PSRSOperandMixin):
     @classmethod
     def _tile_psrs(cls, op, in_data):
         out = op.outputs[0]
-        in_df, axis_chunk_shape, _, _ = cls.preprocess(op, in_data=in_data)
+        in_df, axis_chunk_shape, _, _ = yield from cls.preprocess(op, in_data=in_data)
 
         # stage 1: local sort and regular samples collected
         sorted_chunks, _, sampled_chunks = cls.local_sort_and_regular_sample(
@@ -288,12 +298,12 @@ class DataFramePSRSChunkOperand(DataFrameOperand):
     _n_partition = Int32Field('n_partition')
 
     def __init__(self, sort_type=None, by=None, axis=None, ascending=None, inplace=None, kind=None,
-                 na_position=None, level=None, sort_remaining=None, n_partition=None, gpu=None,
+                 na_position=None, level=None, sort_remaining=None, n_partition=None,
                  output_types=None, **kw):
         super().__init__(_sort_type=sort_type, _by=by, _axis=axis, _ascending=ascending,
                          _inplace=inplace, _kind=kind, _na_position=na_position,
                          _level=level, _sort_remaining=sort_remaining, _n_partition=n_partition,
-                         _gpu=gpu, _output_types=output_types, **kw)
+                         _output_types=output_types, **kw)
 
     @property
     def sort_type(self):
@@ -359,8 +369,7 @@ class DataFramePSRSSortRegularSample(DataFramePSRSChunkOperand, DataFrameOperand
             ctx[op.outputs[0].key] = res = execute_sort_index(a, op)
 
         by = op.by
-        add_distinct_col = bool(int(os.environ.get('PSRS_DISTINCT_COL', '0'))) \
-            or getattr(ctx, 'running_mode', None) == RunningMode.distributed
+        add_distinct_col = bool(int(os.environ.get('PSRS_DISTINCT_COL', '0')))
         if add_distinct_col and isinstance(a, xdf.DataFrame) and op.sort_type == 'sort_values':
             # when running under distributed mode, we introduce an extra column
             # to make sure pivots are distinct
@@ -424,7 +433,7 @@ class DataFramePSRSConcatPivot(DataFramePSRSChunkOperand, DataFrameOperandMixin)
             ctx[op.outputs[-1].key] = a.index[slc]
 
 
-class DataFramePSRSShuffle(DataFrameMapReduceOperand, DataFrameOperandMixin):
+class DataFramePSRSShuffle(MapReduceOperand, DataFrameOperandMixin):
     _op_type_ = OperandDef.PSRS_SHUFFLE
 
     _sort_type = StringField('sort_type')
@@ -446,11 +455,11 @@ class DataFramePSRSShuffle(DataFrameMapReduceOperand, DataFrameOperandMixin):
 
     def __init__(self, sort_type=None, by=None, axis=None, ascending=None, n_partition=None,
                  na_position=None, inplace=None, kind=None, level=None, sort_remaining=None,
-                 stage=None, shuffle_key=None, output_types=None, **kw):
+                 output_types=None, **kw):
         super().__init__(_sort_type=sort_type, _by=by, _axis=axis, _ascending=ascending,
                          _n_partition=n_partition, _na_position=na_position, _inplace=inplace,
-                         _kind=kind, _level=level, _sort_remaining=sort_remaining, _stage=stage,
-                         _shuffle_key=shuffle_key, _output_types=output_types, **kw)
+                         _kind=kind, _level=level, _sort_remaining=sort_remaining,
+                         _output_types=output_types, **kw)
 
     @property
     def sort_type(self):
@@ -515,7 +524,7 @@ class DataFramePSRSShuffle(DataFrameMapReduceOperand, DataFrameOperandMixin):
         if len(a) == 0:
             # when the chunk is empty, no slices can be produced
             for i in range(op.n_partition):
-                ctx[(out.key, str(i))] = a
+                ctx[out.key, (i,)] = a
             return
 
         # use numpy.searchsorted to find split positions.
@@ -535,7 +544,9 @@ class DataFramePSRSShuffle(DataFrameMapReduceOperand, DataFrameOperandMixin):
         poses = (None,) + tuple(poses) + (None,)
         for i in range(op.n_partition):
             values = a.iloc[poses[i]: poses[i + 1]]
-            ctx[(out.key, str(i))] = values
+            if is_cudf(values):  # pragma: no cover
+                values = values.copy()
+            ctx[out.key, (i,)] = values
 
     @classmethod
     def _calc_series_poses(cls, s, pivots, ascending=True):
@@ -553,7 +564,7 @@ class DataFramePSRSShuffle(DataFrameMapReduceOperand, DataFrameOperandMixin):
         if len(a) == 0:
             # when the chunk is empty, no slices can be produced
             for i in range(op.n_partition):
-                ctx[(out.key, str(i))] = a
+                ctx[out.key, (i,)] = a
             return
 
         if isinstance(a, pd.Series):
@@ -566,7 +577,7 @@ class DataFramePSRSShuffle(DataFrameMapReduceOperand, DataFrameOperandMixin):
             poses = (None,) + tuple(poses) + (None,)
             for i in range(op.n_partition):
                 values = a.iloc[poses[i]: poses[i + 1]]
-                ctx[(out.key, str(i))] = values
+                ctx[out.key, (i,)] = values
 
     @classmethod
     def _execute_sort_index_map(cls, ctx, op):
@@ -580,7 +591,7 @@ class DataFramePSRSShuffle(DataFrameMapReduceOperand, DataFrameOperandMixin):
         poses = (None,) + tuple(poses) + (None,)
         for i in range(op.n_partition):
             values = a.iloc[poses[i]: poses[i + 1]]
-            ctx[(out.key, str(i))] = values
+            ctx[out.key, (i,)] = values
 
     @classmethod
     def _execute_map(cls, ctx, op):
@@ -596,13 +607,9 @@ class DataFramePSRSShuffle(DataFrameMapReduceOperand, DataFrameOperandMixin):
             cls._execute_sort_index_map(ctx, op)
 
     @classmethod
-    def _execute_reduce(cls, ctx, op):
+    def _execute_reduce(cls, ctx, op: "DataFramePSRSShuffle"):
         out_chunk = op.outputs[0]
-        input_keys, _ = get_shuffle_input_keys_idxes(op.inputs[0])
-        if getattr(ctx, 'running_mode', None) == RunningMode.distributed:
-            raw_inputs = [ctx.pop((input_key, op.shuffle_key)) for input_key in input_keys]
-        else:
-            raw_inputs = [ctx[(input_key, op.shuffle_key)] for input_key in input_keys]
+        raw_inputs = list(op.iter_mapper_data(ctx, pop=False))
 
         xdf = pd if isinstance(raw_inputs[0], (pd.DataFrame, pd.Series)) else cudf
         if xdf is pd:
