@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,15 +24,17 @@ from pandas.api.types import is_string_dtype
 from pandas.api.extensions import ExtensionDtype
 from pandas.core.dtypes.cast import find_common_type
 
-try:
-    import pyarrow as pa
-except ImportError:  # pragma: no cover
-    pass
-
 from ..core import Entity, ExecutableTuple
 from ..lib.mmh3 import hash as mmh_hash
 from ..tensor.utils import dictify_chunk_size, normalize_chunk_sizes
-from ..utils import tokenize, sbytes
+from ..utils import tokenize, sbytes, lazy_import, ModulePlaceholder
+
+try:
+    import pyarrow as pa
+except ImportError:  # pragma: no cover
+    pa = ModulePlaceholder('pyarrow')
+
+cudf = lazy_import('cudf', globals=globals(), rename='cudf')
 
 
 def hash_index(index, size):
@@ -49,6 +51,8 @@ def hash_dataframe_on(df, on, size, level=None):
         idx = df.index
         if level is not None:
             idx = idx.to_frame(False)[level]
+        if cudf and isinstance(idx, cudf.Index):  # pragma: no cover
+            idx = idx.to_pandas()
         hashed_label = pd.util.hash_pandas_object(idx, categorize=False)
     elif callable(on):
         # todo optimization can be added, if ``on`` is a numpy ufunc or sth can be vectorized
@@ -80,6 +84,22 @@ def sort_dataframe_inplace(df, *axis):
     return df
 
 
+@functools.lru_cache(1)
+def _get_range_index_type():
+    if cudf is not None:
+        return pd.RangeIndex, cudf.RangeIndex
+    else:
+        return pd.RangeIndex
+
+
+@functools.lru_cache(1)
+def _get_multi_index_type():
+    if cudf is not None:
+        return pd.MultiIndex, cudf.MultiIndex
+    else:
+        return pd.MultiIndex
+
+
 def _get_range_index_start(pd_range_index):
     try:
         return pd_range_index.start
@@ -98,7 +118,11 @@ def _get_range_index_step(pd_range_index):
     try:
         return pd_range_index.step
     except AttributeError:  # pragma: no cover
+        pass
+    try:  # pragma: no cover
         return pd_range_index._step
+    except AttributeError:  # pragma: no cover
+        return 1  # cudf does not support step arg
 
 
 def is_pd_range_empty(pd_range_index):
@@ -293,9 +317,13 @@ def parse_index(index_value, *args, store_data=False, key=None):
             _max_val_close=True,
             _key=key or tokenize(*args),
         ))
-    if isinstance(index_value, pd.RangeIndex):
+    if hasattr(index_value, 'to_pandas'):  # pragma: no cover
+        # convert cudf.Index to pandas
+        index_value = index_value.to_pandas()
+
+    if isinstance(index_value, _get_range_index_type()):
         return IndexValue(_index_value=_serialize_range_index(index_value))
-    elif isinstance(index_value, pd.MultiIndex):
+    elif isinstance(index_value, _get_multi_index_type()):
         return IndexValue(_index_value=_serialize_multi_index(index_value))
     else:
         return IndexValue(_index_value=_serialize_index(index_value))
@@ -583,7 +611,9 @@ def build_series(series_obj, fill_value=1, size=1, name=None, ensure_string=Fals
 def concat_index_value(index_values, store_data=False):
     if not isinstance(index_values, (list, tuple)):
         index_values = [index_values]
-    result = index_values[0].value.to_pandas()
+    result = index_values[0]
+    if not isinstance(result, pd.Index):
+        result = result.to_pandas()
     for index_value in index_values[1:]:
         if isinstance(index_value, pd.Index):
             result = result.append(index_value)
@@ -596,7 +626,7 @@ def build_concatenated_rows_frame(df):
     from ..core import OutputType
     from .merge.concat import DataFrameConcat
 
-    # When the df isn't splitted along the column axis, return the df directly.
+    # When the df isn't split along the column axis, return the df directly.
     if df.chunk_shape[1] == 1:
         return df
 
@@ -976,7 +1006,8 @@ def fetch_corner_data(df_or_series, session=None) -> pd.DataFrame:
         tail = iloc(df_or_series)[-index_size:]
         head_data, tail_data = \
             ExecutableTuple([head, tail]).fetch(session=session)
-        return pd.concat([head_data, tail_data], axis='index')
+        xdf = cudf if head.op.is_gpu() else pd
+        return xdf.concat([head_data, tail_data], axis='index')
 
 
 class ReprSeries(pd.Series):
@@ -1115,3 +1146,38 @@ def make_dtypes(dtypes):
         else:
             dtypes = pd.Series(dtypes)
     return dtypes.apply(make_dtype)
+
+
+def is_dataframe(x):
+    if cudf is not None:  # pragma: no cover
+        if isinstance(x, cudf.DataFrame):
+            return True
+    return isinstance(x, pd.DataFrame)
+
+
+def is_series(x):
+    if cudf is not None:  # pragma: no cover
+        if isinstance(x, cudf.Series):
+            return True
+    return isinstance(x, pd.Series)
+
+
+def is_index(x):
+    if cudf is not None:  # pragma: no cover
+        if isinstance(x, cudf.Index):
+            return True
+    return isinstance(x, pd.Index)
+
+
+def get_xdf(x):
+    if cudf is not None:  # pragma: no cover
+        if isinstance(x, (cudf.DataFrame, cudf.Series, cudf.Index)):
+            return cudf
+    return pd
+
+
+def is_cudf(x):
+    if cudf is not None:  # pragma: no cover
+        if isinstance(x, (cudf.DataFrame, cudf.Series, cudf.Index)):
+            return True
+    return False
