@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,8 +19,8 @@ from typing import Dict
 import numpy as np
 import pandas as pd
 
-from ...operands import OperandStage
-from ...serialize import AnyField, BoolField, Int32Field, Int64Field, \
+from ...core.operand import OperandStage
+from ...serialization.serializables import AnyField, BoolField, Int32Field, Int64Field, \
     DictField, StringField
 from ...utils import tokenize
 from ..core import DATAFRAME_TYPE
@@ -59,8 +59,8 @@ class BaseDataFrameExpandingAgg(DataFrameOperand, DataFrameOperandMixin):
     def __init__(self, min_periods=None, axis=None, func=None, count_always_valid=None,
                  append_index=None, output_agg=False, map_groups=None, map_sources=None,
                  combine_sources=None, combine_columns=None, combine_funcs=None,
-                 key_to_funcs=None, min_periods_func_name=None, stage=None, **kw):
-        super().__init__(_min_periods=min_periods, _axis=axis, _func=func, _stage=stage,
+                 key_to_funcs=None, min_periods_func_name=None, **kw):
+        super().__init__(_min_periods=min_periods, _axis=axis, _func=func,
                          _count_always_valid=count_always_valid, _append_index=append_index,
                          _output_agg=output_agg, _map_groups=map_groups, _map_sources=map_sources,
                          _combine_sources=combine_sources, _combine_columns=combine_columns,
@@ -334,7 +334,7 @@ class BaseDataFrameExpandingAgg(DataFrameOperand, DataFrameOperandMixin):
 
             chunk_op = op.copy().reset_key()
             chunk_op._output_agg = c.index[axis] != in_df.chunk_shape[axis] - 1
-            chunk_op._stage = OperandStage.map
+            chunk_op.stage = OperandStage.map
             chunk_op._map_sources = stage_info.map_sources
             chunk_op._map_groups = stage_info.map_groups
             chunk_op._key_to_funcs = stage_info.key_to_funcs
@@ -362,7 +362,7 @@ class BaseDataFrameExpandingAgg(DataFrameOperand, DataFrameOperandMixin):
 
             chunk_op = op.copy().reset_key()
             chunk_op._output_agg = False
-            chunk_op._stage = OperandStage.combine
+            chunk_op.stage = OperandStage.combine
             chunk_op._map_groups = stage_info.map_groups
             chunk_op._combine_sources = stage_info.combine_sources
             chunk_op._combine_columns = stage_info.combine_columns
@@ -480,7 +480,11 @@ class BaseDataFrameExpandingAgg(DataFrameOperand, DataFrameOperandMixin):
             for func_name in func_to_aggs.keys():
                 if func_name == 'count':
                     if not op.count_always_valid and pred_record_count < op.min_periods - 1:
-                        func_to_aggs[func_name].iloc[:op.min_periods - pred_record_count - 1] = np.nan
+                        try:
+                            func_to_aggs[func_name].iloc[:op.min_periods - pred_record_count - 1] = np.nan
+                        except ValueError:
+                            func_to_aggs[func_name] = func_to_aggs[func_name].copy()
+                            func_to_aggs[func_name].iloc[:op.min_periods - pred_record_count - 1] = np.nan
                 else:
                     func_to_aggs[func_name][invalid_poses] = np.nan
 
