@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,20 +13,60 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import abc
+import functools
+import re
+
 from ... import __version__ as mars_version
-from ...utils import parse_readable_size
+from ...utils import parse_readable_size, calc_size_by_str
 
 DEFAULT_IMAGE = 'marsproject/mars:v' + mars_version
+DEFAULT_WORKER_CACHE_MEM = '40%'
 
 
 def _remove_nones(cfg):
     return dict((k, v) for k, v in cfg.items() if v is not None)
 
 
-class RoleConfig:
+_kube_api_mapping = {
+    'v1': 'CoreV1Api',
+    'apps/v1': 'AppsV1Api',
+    'rbac.authorization.k8s.io/v1': 'RbacAuthorizationV1Api',
+}
+
+
+@functools.lru_cache(10)
+def _get_k8s_api(api_version, k8s_api_client):
+    from kubernetes import client as kube_client
+    return getattr(kube_client, _kube_api_mapping[api_version])(k8s_api_client)
+
+
+@functools.lru_cache(10)
+def _camel_to_underline(name):
+    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
+    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()
+
+
+class KubeConfig(abc.ABC):
+    api_version = 'v1'
+
+    def create_namespaced(self, k8s_api_client, namespace):
+        api = _get_k8s_api(self.api_version, k8s_api_client)
+        config = self.build()
+        method_name = f'create_namespaced_{_camel_to_underline(config["kind"])}'
+        return getattr(api, method_name)(namespace, config)
+
+    @abc.abstractmethod
+    def build(self):
+        """Build config dict of the object"""
+
+
+class RoleConfig(KubeConfig):
     """
     Configuration builder for Kubernetes RBAC roles
     """
+    api_version = 'rbac.authorization.k8s.io/v1'
+
     def __init__(self, name, namespace, api_groups, resources, verbs):
         self._name = name
         self._namespace = namespace
@@ -46,10 +86,12 @@ class RoleConfig:
         }
 
 
-class RoleBindingConfig:
+class RoleBindingConfig(KubeConfig):
     """
     Configuration builder for Kubernetes RBAC role bindings
     """
+    api_version = 'rbac.authorization.k8s.io/v1'
+
     def __init__(self, name, namespace, role_name, service_account_name):
         self._name = name
         self._namespace = namespace
@@ -73,7 +115,7 @@ class RoleBindingConfig:
         }
 
 
-class NamespaceConfig:
+class NamespaceConfig(KubeConfig):
     """
     Configuration builder for Kubernetes namespaces
     """
@@ -92,7 +134,7 @@ class NamespaceConfig:
         }
 
 
-class ServiceConfig:
+class ServiceConfig(KubeConfig):
     """
     Configuration builder for Kubernetes services
     """
@@ -110,6 +152,9 @@ class ServiceConfig:
             'kind': 'Service',
             'metadata': {
                 'name': self._name,
+                'labels': {
+                    'mars/service-name': self._name,
+                },
             },
             'spec': _remove_nones({
                 'type': self._type,
@@ -143,10 +188,10 @@ class ResourceConfig:
         return self._memory
 
     def build(self):
-        return {
-            'cpu': f'{int(self._cpu * 1000)}m',
-            'memory': str(int(self._memory)),
-        }
+        return _remove_nones({
+            'cpu': f'{int(self._cpu * 1000)}m' if self._cpu else None,
+            'memory': str(int(self._memory)) if self._memory else None,
+        })
 
 
 class PortConfig:
@@ -162,7 +207,7 @@ class PortConfig:
         }
 
 
-class VolumeConfig:
+class VolumeConfig(abc.ABC):
     """
     Base configuration builder for Kubernetes volumes
     """
@@ -170,8 +215,9 @@ class VolumeConfig:
         self.name = name
         self.mount_path = mount_path
 
+    @abc.abstractmethod
     def build(self):
-        raise NotImplementedError
+        """Build volume config"""
 
     def build_mount(self):
         return {
@@ -200,9 +246,10 @@ class EmptyDirVolumeConfig(VolumeConfig):
     """
     Configuration builder for Kubernetes empty-dir volumes
     """
-    def __init__(self, name, mount_path, use_memory=False):
+    def __init__(self, name, mount_path, use_memory=True, size_limit=None):
         super().__init__(name, mount_path)
         self._medium = 'Memory' if use_memory else None
+        self._size_limit = size_limit
 
     def build(self):
         result = {
@@ -211,6 +258,8 @@ class EmptyDirVolumeConfig(VolumeConfig):
         }
         if self._medium:
             result['emptyDir']['medium'] = self._medium
+        if self._size_limit:
+            result['emptyDir']['sizeLimit'] = str(int(self._size_limit))
         return result
 
 
@@ -254,45 +303,31 @@ class ProbeConfig:
         })
 
 
-class ExecProbeConfig(ProbeConfig):
+class TcpSocketProbeConfig(ProbeConfig):
     """
-    Configuration builder for Kubernetes probes by executing commands
+    Configuration builder for TCP liveness and readiness probes
     """
-    def __init__(self, command, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self._command = command
-
-    def build(self):
-        result = {
-            'exec': {'command': self._command}
-        }
-        result.update(super().build())
-        return result
-
-
-class TcpProbeConfig(ProbeConfig):
-    """
-    Configuration builder for Kubernetes probes by checking TCP ports
-    """
-    def __init__(self, port, *args, **kwargs):
-        super().__init__(*args, **kwargs)
+    def __init__(self, port: int, **kwargs):
+        super().__init__(**kwargs)
         self._port = port
 
     def build(self):
-        result = {
-            'tcpSocket': {'port': self._port}
-        }
-        result.update(super().build())
-        return result
+        ret = super().build()
+        ret['tcpSocket'] = {'port': self._port}
+        return ret
 
 
-class ReplicationControllerConfig:
+class ReplicationConfig(KubeConfig):
     """
     Base configuration builder for Kubernetes replication controllers
     """
+    _default_kind = 'Deployment'
+
     def __init__(self, name, image, replicas, resource_request=None, resource_limit=None,
-                 liveness_probe=None, readiness_probe=None, pre_stop_command=None):
+                 liveness_probe=None, readiness_probe=None, pre_stop_command=None,
+                 kind=None):
         self._name = name
+        self._kind = kind or self._default_kind
         self._image = image
         self._replicas = replicas
         self._ports = []
@@ -310,6 +345,10 @@ class ReplicationControllerConfig:
 
         self._pre_stop_command = pre_stop_command
 
+    @property
+    def api_version(self):
+        return 'apps/v1' if self._kind in ('Deployment', 'ReplicaSet') else 'v1'
+
     def add_env(self, name, value=None, field_path=None):
         self._envs[name] = ContainerEnvConfig(name, value=value, field_path=field_path)
 
@@ -332,8 +371,9 @@ class ReplicationControllerConfig:
     def add_volume(self, vol):
         self._volumes.append(vol)
 
+    @abc.abstractmethod
     def build_container_command(self):
-        raise NotImplementedError
+        """Output container command"""
 
     def build_container(self):
         resources_dict = {
@@ -367,7 +407,7 @@ class ReplicationControllerConfig:
 
     def build(self):
         return {
-            'kind': 'ReplicationController',
+            'kind': self._kind,
             'metadata': {
                 'name': self._name,
             },
@@ -383,15 +423,16 @@ class ReplicationControllerConfig:
         }
 
 
-class MarsReplicationControllerConfig(ReplicationControllerConfig):
+class MarsReplicationConfig(ReplicationConfig, abc.ABC):
     """
     Base configuration builder for replication controllers for Mars
     """
     rc_name = None
+    default_readiness_port = 15031
 
     def __init__(self, replicas, cpu=None, memory=None, limit_resources=False,
                  memory_limit_ratio=None, image=None, modules=None, volumes=None,
-                 service_port=None, **kwargs):
+                 service_name=None, service_port=None, **kwargs):
         self._cpu = cpu
         self._memory, ratio = parse_readable_size(memory) if memory is not None else (None, False)
         assert not ratio
@@ -405,6 +446,7 @@ class MarsReplicationControllerConfig(ReplicationControllerConfig):
         limit_res = ResourceConfig(
             req_res.cpu, req_res.memory * (memory_limit_ratio or 1)) if req_res and memory else None
 
+        self._service_name = service_name
         self._service_port = service_port
 
         super().__init__(
@@ -425,6 +467,8 @@ class MarsReplicationControllerConfig(ReplicationControllerConfig):
         self.add_env('MARS_K8S_POD_NAMESPACE', field_path='metadata.namespace')
         self.add_env('MARS_K8S_POD_IP', field_path='status.podIP')
 
+        if self._service_name:
+            self.add_env('MARS_K8S_SERVICE_NAME', str(self._service_name))
         if self._service_port:
             self.add_env('MARS_K8S_SERVICE_PORT', str(self._service_port))
 
@@ -451,32 +495,42 @@ class MarsReplicationControllerConfig(ReplicationControllerConfig):
 
     def build(self):
         result = super().build()
-        result['spec']['selector'] = {'mars/service-type': self.rc_name}
+        if self._kind in ('Deployment', 'ReplicaSet'):
+            result['spec']['selector'] = {'matchLabels': {'mars/service-type': self.rc_name}}
+        else:
+            result['spec']['selector'] = {'mars/service-type': self.rc_name}
         return result
 
 
-class MarsSchedulersConfig(MarsReplicationControllerConfig):
+class MarsSupervisorsConfig(MarsReplicationConfig):
     """
-    Configuration builder for Mars scheduler service
+    Configuration builder for Mars supervisor service
     """
-    rc_name = 'marsscheduler'
+    rc_name = 'marssupervisor'
+
+    def __init__(self, *args, **kwargs):
+        self._web_port = kwargs.pop('web_port', None)
+        self._readiness_port = kwargs.pop('readiness_port', self.default_readiness_port)
+        super().__init__(*args, **kwargs)
+        if self._web_port:
+            self.add_port(self._web_port)
 
     def config_readiness_probe(self):
-        readiness_cmd = [
-            '/srv/entrypoint.sh', self.get_local_app_module('probe'),
-        ]
-        return ExecProbeConfig(readiness_cmd, timeout=60, failure_thresh=10)
+        return TcpSocketProbeConfig(
+            self._readiness_port, timeout=60, failure_thresh=10)
 
     def build_container_command(self):
         cmd = [
-            '/srv/entrypoint.sh', self.get_local_app_module('scheduler'),
+            '/srv/entrypoint.sh', self.get_local_app_module('supervisor'),
         ]
         if self._service_port:
             cmd += ['-p', str(self._service_port)]
+        if self._web_port:
+            cmd += ['-w', str(self._web_port)]
         return cmd
 
 
-class MarsWorkersConfig(MarsReplicationControllerConfig):
+class MarsWorkersConfig(MarsReplicationConfig):
     """
     Configuration builder for Mars worker service
     """
@@ -486,8 +540,10 @@ class MarsWorkersConfig(MarsReplicationControllerConfig):
         spill_volumes = kwargs.pop('spill_volumes', None) or ()
         mount_shm = kwargs.pop('mount_shm', True)
         self._limit_resources = kwargs['limit_resources'] = kwargs.get('limit_resources', True)
-        worker_cache_mem = kwargs.pop('worker_cache_mem', None)
+        worker_cache_mem = kwargs.pop('worker_cache_mem', None) or DEFAULT_WORKER_CACHE_MEM
         min_cache_mem = kwargs.pop('min_cache_mem', None)
+        self._readiness_port = kwargs.pop('readiness_port', self.default_readiness_port)
+        supervisor_web_port = kwargs.pop('supervisor_web_port', None)
 
         super().__init__(*args, **kwargs)
 
@@ -503,19 +559,25 @@ class MarsWorkersConfig(MarsReplicationControllerConfig):
         if self._spill_volumes:
             self.add_env('MARS_SPILL_DIRS', ':'.join(self._spill_volumes))
 
-        if mount_shm:
-            self.add_env('MARS_K8S_REMOUNT_SHM', '1')
-
-        if worker_cache_mem:
+        if self._memory:
+            size_limit = calc_size_by_str(worker_cache_mem, self._memory)
             self.add_env('MARS_CACHE_MEM_SIZE', worker_cache_mem)
+        else:
+            size_limit = None
+
+        if mount_shm:
+            self.add_volume(EmptyDirVolumeConfig(
+                'mars-shared', '/dev/shm', size_limit=size_limit
+            ))
+
         if min_cache_mem:
             self.add_env('MARS_MIN_CACHE_MEM_SIZE', min_cache_mem)
+        if supervisor_web_port:
+            self.add_env('MARS_K8S_SUPERVISOR_WEB_PORT', supervisor_web_port)
 
     def config_readiness_probe(self):
-        readiness_cmd = [
-            '/srv/entrypoint.sh', self.get_local_app_module('probe'),
-        ]
-        return ExecProbeConfig(readiness_cmd, timeout=60, failure_thresh=10)
+        return TcpSocketProbeConfig(
+            self._readiness_port, timeout=60, failure_thresh=10)
 
     def build_container_command(self):
         cmd = [
@@ -524,22 +586,3 @@ class MarsWorkersConfig(MarsReplicationControllerConfig):
         if self._service_port:
             cmd += ['-p', str(self._service_port)]
         return cmd
-
-
-class MarsWebsConfig(MarsReplicationControllerConfig):
-    """
-    Configuration builder for Mars web service
-    """
-    rc_name = 'marsweb'
-
-    def config_readiness_probe(self):
-        if self._service_port:
-            return TcpProbeConfig(self._service_port, timeout=60, failure_thresh=10)
-
-    def build_container_command(self):
-        cmd = [
-            '/srv/entrypoint.sh', self.get_local_app_module('web'),
-        ]
-        if self._service_port:
-            cmd += ['-p', str(self._service_port)]
-        return cmd
