@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,15 +15,16 @@
 import warnings
 
 import numpy as np
-from sklearn.base import BaseEstimator, ClusterMixin, TransformerMixin
+from sklearn.base import ClusterMixin, TransformerMixin
 from sklearn.exceptions import ConvergenceWarning
 
 from ... import tensor as mt
 from ...tensor.utils import check_random_state
+from ..base import BaseEstimator
 from ..metrics.pairwise import euclidean_distances
 from ..utils.extmath import row_norms
 from ..utils.validation import _check_sample_weight, check_array, \
-    check_X_y, _num_samples, check_is_fitted
+    _num_samples, check_is_fitted
 from ._k_means_common import _inertia
 from ._k_means_elkan_iter import init_bounds, elkan_iter
 from ._k_means_init import _k_init, _scalable_k_init
@@ -180,19 +181,12 @@ def k_means(X, n_clusters, sample_weight=None, init='k-means||',
         return est.cluster_centers_, est.labels_, est.inertia_
 
 
-def _kmeans_single_elkan(X, sample_weight, n_clusters, max_iter=300,
-                         init='k-means++', verbose=False, x_squared_norms=None,
-                         random_state=None, tol=1e-4, oversampling_factor=2,
-                         init_iter=5, X_mean=None, session=None, run_kwargs=None):
-    random_state = check_random_state(random_state)
+def _kmeans_single_elkan(X, sample_weight, centers_init, n_clusters, max_iter=300,
+                         verbose=False, x_squared_norms=None,
+                         tol=1e-4, X_mean=None, session=None, run_kwargs=None):
     sample_weight = _check_normalize_sample_weight(sample_weight, X)
 
-    # init
-    centers = _init_centroids(X, n_clusters, init, random_state=random_state,
-                              x_squared_norms=x_squared_norms,
-                              oversampling_factor=oversampling_factor,
-                              init_iter=init_iter)
-
+    centers = centers_init
     # execute X, centers and tol first
     tol = mt.asarray(tol)
     to_run = [X, sample_weight, centers, x_squared_norms, tol]
@@ -266,19 +260,12 @@ def _kmeans_single_elkan(X, sample_weight, n_clusters, max_iter=300,
     return labels, inertia, centers, i + 1
 
 
-def _kmeans_single_lloyd(X, sample_weight, n_clusters, max_iter=300,
-                         init='k-means++', verbose=False, x_squared_norms=None,
-                         random_state=None, tol=1e-4, oversampling_factor=2,
-                         init_iter=5, X_mean=None, session=None, run_kwargs=None):
-    random_state = check_random_state(random_state)
+def _kmeans_single_lloyd(X, sample_weight, centers_init, n_clusters, max_iter=300,
+                         verbose=False, x_squared_norms=None,
+                         tol=1e-4, X_mean=None, session=None, run_kwargs=None):
     sample_weight = _check_normalize_sample_weight(sample_weight, X)
 
-    # init
-    centers = _init_centroids(X, n_clusters, init, random_state=random_state,
-                              x_squared_norms=x_squared_norms,
-                              oversampling_factor=oversampling_factor,
-                              init_iter=init_iter)
-
+    centers = centers_init
     # execute X, centers and tol first
     tol = mt.asarray(tol)
     to_run = [X, centers, x_squared_norms, tol]
@@ -622,6 +609,60 @@ class KMeans(TransformerMixin, ClusterMixin, BaseEstimator):
         self.oversampling_factor = oversampling_factor
         self.init_iter = init_iter
 
+    def _check_params(self, X):
+        # n_init
+        if self.n_init <= 0:
+            raise ValueError(
+                f"n_init should be > 0, got {self.n_init} instead.")
+        self._n_init = self.n_init
+
+        # max_iter
+        if self.max_iter <= 0:
+            raise ValueError(
+                f"max_iter should be > 0, got {self.max_iter} instead.")
+
+        # n_clusters
+        if X.shape[0] < self.n_clusters:
+            raise ValueError(f"n_samples={X.shape[0]} should be >= "
+                             f"n_clusters={self.n_clusters}.")
+
+        # tol
+        self._tol = _tolerance(X, self.tol)
+
+        # algorithm
+        if self.algorithm not in ("auto", "full", "elkan"):
+            raise ValueError(f"Algorithm must be 'auto', 'full' or 'elkan', "
+                             f"got {self.algorithm} instead.")
+
+        self._algorithm = self.algorithm
+        if self._algorithm == "auto":
+            # note(xuye.qin):
+            # Different from scikit-learn,
+            # for now, full seems more efficient when data is large,
+            # elkan needs to be tuned more
+            # old: algorithm = "full" if self.n_clusters == 1 else "elkan"
+            self._algorithm = "full"
+        if self._algorithm == "elkan" and self.n_clusters == 1:
+            warnings.warn("algorithm='elkan' doesn't make sense for a single "
+                          "cluster. Using 'full' instead.", RuntimeWarning)
+            self._algorithm = "full"
+
+        # init
+        if not (hasattr(self.init, '__array__') or callable(self.init)
+                or (isinstance(self.init, str)
+                    and self.init in ["k-means++", "k-means||", "random"])):
+            raise ValueError(
+                f"init should be either 'k-means++'， 'k-mean||', 'random', "
+                f"a tensor, a ndarray or a "
+                f"callable, got '{self.init}' instead.")
+
+        if hasattr(self.init, '__array__') and self._n_init != 1:
+            warnings.warn(
+                f"Explicit initial center position passed: performing only"
+                f" one init in {self.__class__.__name__} instead of "
+                f"n_init={self._n_init}.", RuntimeWarning, stacklevel=2)
+            self._n_init = 1
+
     def _check_test_data(self, X):
         X = check_array(X, accept_sparse=True, dtype=[np.float64, np.float32],
                         order='C', accept_large_sparse=False)
@@ -633,65 +674,6 @@ class KMeans(TransformerMixin, ClusterMixin, BaseEstimator):
 
         return X
 
-    def _validate_data(self, X, y=None, reset=True,
-                       validate_separately=False, **check_params):
-        """Validate input data and set or check the `n_features_in_` attribute.
-
-        Parameters
-        ----------
-        X : {array-like, sparse matrix, dataframe} of shape \
-                (n_samples, n_features)
-            The input samples.
-        y : array-like of shape (n_samples,), default=None
-            The targets. If None, `check_array` is called on `X` and
-            `check_X_y` is called otherwise.
-        reset : bool, default=True
-            Whether to reset the `n_features_in_` attribute.
-            If False, the input will be checked for consistency with data
-            provided when reset was last True.
-        validate_separately : False or tuple of dicts, default=False
-            Only used if y is not None.
-            If False, call validate_X_y(). Else, it must be a tuple of kwargs
-            to be used for calling check_array() on X and y respectively.
-        **check_params : kwargs
-            Parameters passed to :func:`sklearn.utils.check_array` or
-            :func:`sklearn.utils.check_X_y`. Ignored if validate_separately
-            is not False.
-
-        Returns
-        -------
-        out : tensor or tuple of these
-            The validated input. A tuple is returned if `y` is not None.
-        """
-
-        if y is None:
-            if hasattr(self, '_get_tags') and \
-                    self._get_tags().get('requires_y', False):  # pragma: no cover
-                raise ValueError(
-                    f"This {type(self).__name__} estimator requires y to be passed, "
-                    "but the target y is None."
-                )
-            X = check_array(X, **check_params)
-            out = X
-        else:  # pragma: no cover
-            if validate_separately:
-                # We need this because some estimators validate X and y
-                # separately, and in general, separately calling check_array()
-                # on X and y isn't equivalent to just calling check_X_y()
-                # :(
-                check_X_params, check_y_params = validate_separately
-                X = check_array(X, **check_X_params)
-                y = check_array(y, **check_y_params)
-            else:
-                X, y = check_X_y(X, y, **check_params)
-            out = X, y
-
-        if check_params.get('ensure_2d', True) and \
-                hasattr(self, '_check_n_features'):
-            self._check_n_features(X, reset=reset)
-
-        return out
-
     def fit(self, X, y=None, sample_weight=None, session=None, run_kwargs=None):
         """Compute k-means clustering.
 
@@ -716,19 +698,6 @@ class KMeans(TransformerMixin, ClusterMixin, BaseEstimator):
         self
             Fitted estimator.
         """
-        random_state = check_random_state(self.random_state).to_numpy()
-
-        n_init = self.n_init
-        if n_init <= 0:
-            raise ValueError("Invalid number of initializations. "
-                             f"n_init={n_init} must be bigger than zero.")
-
-        if self.max_iter <= 0:  # pragma: no cover
-            raise ValueError(
-                'Number of iterations should be a positive number, '
-                f'got {self.max_iter} instead'
-            )
-
         expect_chunk_size_on_columns = mt.tensor(X).shape[1]
         if not np.isnan(expect_chunk_size_on_columns):
             X = mt.tensor(X, chunk_size={1: expect_chunk_size_on_columns})
@@ -738,12 +707,11 @@ class KMeans(TransformerMixin, ClusterMixin, BaseEstimator):
                                 order='C', copy=self.copy_x,
                                 accept_large_sparse=False)
         # verify that the number of samples given is larger than k
-        n_samples = _num_samples(X)
-        if np.isnan(n_samples):  # pragma: no cover
+        if np.isnan(_num_samples(X)):  # pragma: no cover
             X.execute(session=session, **(run_kwargs or dict()))
-            n_samples = _num_samples(X)
-        if n_samples < self.n_clusters:
-            raise ValueError(f"n_samples={n_samples} should be >= n_clusters={self.n_clusters}")
+
+        self._check_params(X)
+        random_state = check_random_state(self.random_state).to_numpy()
 
         tol = _tolerance(X, self.tol)
 
@@ -753,13 +721,6 @@ class KMeans(TransformerMixin, ClusterMixin, BaseEstimator):
             init = check_array(init, dtype=X.dtype.type, copy=True, order='C')
             _validate_center_shape(X, self.n_clusters, init)
 
-            if n_init != 1:  # pragma: no cover
-                warnings.warn(
-                    'Explicit initial center position passed: '
-                    'performing only one init in k-means instead of n_init=%d'
-                    % n_init, RuntimeWarning, stacklevel=2)
-                n_init = 1
-
         # subtract of mean of x for more accurate distance computations
         X_mean = None
         if not X.issparse():
@@ -775,38 +736,24 @@ class KMeans(TransformerMixin, ClusterMixin, BaseEstimator):
 
         best_labels, best_inertia, best_centers = None, None, None
 
-        algorithm = self.algorithm
-        if algorithm == "elkan" and self.n_clusters == 1:  # pragma: no cover
-            warnings.warn("algorithm='elkan' doesn't make sense for a single "
-                          "cluster. Using 'full' instead.", RuntimeWarning)
-            algorithm = "full"
-
-        if algorithm == "auto":
-            # note(xuye.qin):
-            # Different from scikit-learn,
-            # for now, full seems more efficient when data is large,
-            # elkan needs to be tuned more
-            # old: algorithm = "full" if self.n_clusters == 1 else "elkan"
-            algorithm = "full"
-
-        if algorithm == "full":
+        if self._algorithm == "full":
             kmeans_single = _kmeans_single_lloyd
-        elif algorithm == "elkan":
+        else:
             kmeans_single = _kmeans_single_elkan
-        else:  # pragma: no cover
-            raise ValueError(f"Algorithm must be 'auto', 'full' or 'elkan', got {algorithm}")
 
-        # seeds for the initializations of the kmeans runs.
-        seeds = random_state.randint(np.iinfo(np.int32).max, size=n_init)
+        for i in range(self._n_init):  # pylint: disable=unused-variable
+            # Initialize centers
+            centers_init = _init_centroids(
+                X, self.n_clusters, init, random_state=random_state,
+                x_squared_norms=x_squared_norms,
+                oversampling_factor=self.oversampling_factor,
+                init_iter=self.init_iter)
 
-        for seed in seeds:
             # run a k-means once
             labels, inertia, centers, n_iter_ = kmeans_single(
-                X, sample_weight, self.n_clusters, max_iter=self.max_iter,
-                init=init, verbose=self.verbose, tol=tol,
-                x_squared_norms=x_squared_norms, random_state=seed,
-                oversampling_factor=self.oversampling_factor,
-                init_iter=self.init_iter, X_mean=X_mean,
+                X, sample_weight, centers_init, self.n_clusters,
+                max_iter=self.max_iter, verbose=self.verbose, tol=tol,
+                x_squared_norms=x_squared_norms, X_mean=X_mean,
                 session=session, run_kwargs=run_kwargs)
             inertia = inertia.fetch(session=session)
             # determine if these results are the best so far
