@@ -21,8 +21,7 @@ if xgboost:
     from xgboost.sklearn import XGBClassifierBase
 
     from .... import tensor as mt
-    from .dmatrix import MarsDMatrix
-    from .core import evaluation_matrices
+    from .core import wrap_evaluation_matrices
     from .train import train
     from .predict import predict
 
@@ -31,14 +30,16 @@ if xgboost:
         Implementation of the scikit-learn API for XGBoost classification.
         """
 
-        def fit(self, X, y, sample_weights=None, eval_set=None, sample_weight_eval_set=None, **kw):
+        def fit(self, X, y, sample_weight=None, base_margin=None,
+                eval_set=None, sample_weight_eval_set=None, base_margin_eval_set=None, **kw):
             session = kw.pop('session', None)
             run_kwargs = kw.pop('run_kwargs', dict())
             if kw:
                 raise TypeError(f"fit got an unexpected keyword argument '{next(iter(kw))}'")
 
-            dtrain = MarsDMatrix(X, label=y, weight=sample_weights,
-                                 session=session, run_kwargs=run_kwargs)
+            dtrain, evals = wrap_evaluation_matrices(
+                None, X, y, sample_weight, base_margin, eval_set,
+                sample_weight_eval_set, base_margin_eval_set)
             params = self.get_xgb_params()
 
             self.classes_ = mt.unique(y, aggregate_size=1).to_numpy(session=session, **run_kwargs)
@@ -50,8 +51,6 @@ if xgboost:
             else:
                 params['objective'] = 'binary:logistic'
 
-            evals = evaluation_matrices(eval_set, sample_weight_eval_set,
-                                        session=session, run_kwargs=run_kwargs)
             self.evals_result_ = dict()
             result = train(params, dtrain, num_boost_round=self.get_num_boosting_rounds(),
                            evals=evals, evals_result=self.evals_result_,
