@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,12 +18,12 @@ import numpy as np
 import pandas as pd
 
 from .... import opcodes as OperandDef
-from ....serialize import KeyField, BytesField, DictField
+from ....core import recursive_tile
+from ....serialization.serializables import KeyField, BytesField, DictField
 from ....dataframe.core import SERIES_CHUNK_TYPE, DATAFRAME_CHUNK_TYPE
 from ....dataframe.utils import parse_index
 from ....tensor.core import TENSOR_TYPE, TensorOrder
-from ....tiles import TilesError
-from ....utils import check_chunks_unknown_shape
+from ....utils import has_unknown_shape, ensure_own_data
 from ...operands import LearnOperand, LearnOperandMixin, OutputType
 from .dmatrix import ToDMatrix, check_data
 
@@ -86,8 +86,9 @@ class XGBPredict(LearnOperand, LearnOperandMixin):
         out_chunks = []
         data = op.data
         if data.chunk_shape[1] > 1:
-            check_chunks_unknown_shape([op.data], TilesError)
-            data = data.rechunk({1: op.data.shape[1]})._inplace_tile()
+            if has_unknown_shape(op.data):
+                yield
+            data = yield from recursive_tile(data.rechunk({1: op.data.shape[1]}))
         for in_chunk in data.chunks:
             chunk_op = op.copy().reset_key()
             chunk_index = (in_chunk.index[0],)
@@ -130,7 +131,7 @@ class XGBPredict(LearnOperand, LearnOperandMixin):
 
         raw_data = data = ctx[op.data.key]
         if isinstance(data, tuple):
-            data = ToDMatrix.get_xgb_dmatrix(data)
+            data = ToDMatrix.get_xgb_dmatrix(ensure_own_data(data))
         else:
             data = data.spmatrix if hasattr(data, 'spmatrix') else data
             data = DMatrix(data)
