@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,7 +23,7 @@ from sklearn.utils.extmath import fast_logdet
 
 from ... import tensor as mt
 from ... import remote as mr
-from ...core import Base, Entity
+from ...core import ENTITY_TYPE
 from ...tensor.array_utils import get_array_module
 from ...tensor.core import TENSOR_TYPE
 from ...tensor.utils import check_random_state
@@ -111,7 +111,7 @@ def _infer_dimension(spectrum, n_samples):
 
     The returned value will be in [1, n_features - 1].
     """
-    if isinstance(spectrum, (Base, Entity)):
+    if isinstance(spectrum, ENTITY_TYPE):
         spectrum = spectrum.fetch()
     xp = get_array_module(spectrum, nosparse=True)
 
