@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,5 +13,7 @@
 # limitations under the License.
 
 from .pairwise import euclidean_distances, pairwise_distances, pairwise_distances_topk
-from ._classification import accuracy_score
+from ._classification import accuracy_score, log_loss
 from ._ranking import roc_curve, auc
+from ._regresssion import r2_score
+from ._scorer import get_scorer
