@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,12 +15,10 @@
 import cloudpickle
 import numpy as np
 
-from ...context import RunningMode
-from ...core import Object, OBJECT_TYPE, OBJECT_CHUNK_TYPE
-from ...serialize import KeyField, Int32Field, DictField, AnyField, BoolField
-from ...tiles import TilesError
+from ...core import Object, OBJECT_TYPE, OBJECT_CHUNK_TYPE, recursive_tile
+from ...serialization.serializables import KeyField, Int32Field, DictField, AnyField, BoolField
 from ...tensor.core import TensorOrder
-from ...utils import check_chunks_unknown_shape, tokenize
+from ...utils import has_unknown_shape, tokenize
 from ..operands import LearnOperand, LearnOperandMixin, OutputType
 
 
@@ -71,12 +69,13 @@ class TreeBase(LearnOperand, LearnOperandMixin):
 
     @classmethod
     def tile(cls, op):
-        check_chunks_unknown_shape(op.inputs, TilesError)
+        if has_unknown_shape(*op.inputs):
+            yield
 
         # ball tree and kd tree requires the full data,
         # thus rechunk input tensor into 1 chunk
         inp = op.input.rechunk({ax: s for ax, s in enumerate(op.input.shape)})
-        inp = inp._inplace_tile()
+        inp = yield from recursive_tile(inp)
         out = op.outputs[0]
 
         chunk_op = op.copy().reset_key()
@@ -100,12 +99,7 @@ class TreeBase(LearnOperand, LearnOperandMixin):
         tree = cls._tree_type(
             a, op.leaf_size, metric=op.metric,
             **(op.metric_params or dict()))
-        if ctx.running_mode in [RunningMode.local_cluster, RunningMode.distributed]:
-            # for local cluster and distributed, pickle always
-            ctx[op.outputs[0].key] = cloudpickle.dumps(tree)
-        else:
-            # otherwise, to be clear for local, just put into storage directly
-            ctx[op.outputs[0].key] = tree
+        ctx[op.outputs[0].key] = tree
 
 
 def _on_serialize_tree(tree):
@@ -190,8 +184,9 @@ class TreeQueryBase(LearnOperand, LearnOperandMixin):
         inp = op.input
 
         if inp.chunk_shape[1] != 1:
-            check_chunks_unknown_shape([inp], TilesError)
-            inp = inp.rechunk({1: inp.shape[1]})._inplace_tile()
+            if has_unknown_shape(inp):
+                yield
+            inp = yield from recursive_tile(inp.rechunk({1: inp.shape[1]}))
 
         tree_chunk = None
         if isinstance(op.tree, OBJECT_TYPE):
