@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,25 +21,24 @@ import numpy as np
 import pandas as pd
 
 from ... import opcodes as OperandDef
-from ...core import get_output_types
+from ...core import get_output_types, recursive_tile
+from ...core.operand import OperandStage, MapReduceOperand
 from ...dataframe.utils import parse_index
 from ...lib import sparse
-from ...operands import OperandStage
-from ...serialize import ValueType, TupleField, KeyField
+from ...serialization.serializables import FieldTypes, TupleField, KeyField
 from ...tensor.utils import validate_axis, check_random_state, gen_random_seeds, decide_unify_split
 from ...tensor.array_utils import get_array_module
-from ...utils import tokenize, get_shuffle_input_keys_idxes, lazy_import, check_chunks_unknown_shape
-from ...tiles import TilesError
+from ...utils import tokenize, lazy_import, has_unknown_shape
 from ...core import ExecutableTuple
-from ..operands import LearnOperandMixin, OutputType, LearnMapReduceOperand, LearnShuffleProxy
+from ..operands import LearnOperandMixin, OutputType, LearnShuffleProxy
 from ..utils import convert_to_tensor_or_dataframe
 
 
 cudf = lazy_import('cudf')
 
 
-def _shuffle_index_value(op, index_value):
-    key = tokenize((op._values_, index_value.key))
+def _shuffle_index_value(op, index_value, chunk_index=None):
+    key = tokenize((op._values_, chunk_index, index_value.key))
     return parse_index(pd.Index([], index_value.to_pandas().dtype), key=key)
 
 
@@ -50,20 +49,18 @@ def _safe_slice(obj, slc, output_type):
         return obj.iloc[slc]
 
 
-class LearnShuffle(LearnMapReduceOperand, LearnOperandMixin):
+class LearnShuffle(MapReduceOperand, LearnOperandMixin):
     _op_type_ = OperandDef.PERMUTATION
 
-    _axes = TupleField('axes', ValueType.int32)
-    _seeds = TupleField('seeds', ValueType.uint32)
+    _axes = TupleField('axes', FieldTypes.int32)
+    _seeds = TupleField('seeds', FieldTypes.uint32)
 
     _input = KeyField('input')
-    _reduce_sizes = TupleField('reduce_sizes', ValueType.uint32)
+    _reduce_sizes = TupleField('reduce_sizes', FieldTypes.uint32)
 
-    def __init__(self, axes=None, seeds=None, output_types=None, reduce_sizes=None,
-                 stage=None, shuffle_key=None, **kw):
+    def __init__(self, axes=None, seeds=None, output_types=None, reduce_sizes=None, **kw):
         super().__init__(_axes=axes, _seeds=seeds, _output_types=output_types,
-                         _reduce_sizes=reduce_sizes, _stage=stage,
-                         _shuffle_key=shuffle_key, **kw)
+                         _reduce_sizes=reduce_sizes, **kw)
 
     @property
     def axes(self):
@@ -128,7 +125,7 @@ class LearnShuffle(LearnMapReduceOperand, LearnOperandMixin):
             if tuple(tileable.nsplits[ax]) != tuple(nsplit):
                 do_rechunk = True
         if do_rechunk:
-            return tileable.rechunk(ax_nsplit)._inplace_tile()
+            return (yield from recursive_tile(tileable.rechunk(ax_nsplit)))
         else:
             return tileable
 
@@ -156,7 +153,7 @@ class LearnShuffle(LearnMapReduceOperand, LearnOperandMixin):
             else:
                 params['dtypes'] = output.dtypes
                 params['columns_value'] = output.columns_value
-            params['index_value'] = _shuffle_index_value(chunk_op, in_chunk.index_value)
+            params['index_value'] = _shuffle_index_value(chunk_op, in_chunk.index_value, in_chunk.index)
         else:
             assert output_type == OutputType.series
             if no_shuffle:
@@ -164,14 +161,15 @@ class LearnShuffle(LearnMapReduceOperand, LearnOperandMixin):
             else:
                 params['shape'] = (np.nan,)
             params['name'] = in_chunk.name
-            params['index_value'] = _shuffle_index_value(chunk_op, in_chunk.index_value)
+            params['index_value'] = _shuffle_index_value(chunk_op, in_chunk.index_value, in_chunk.index)
             params['dtype'] = in_chunk.dtype
         return params
 
     @classmethod
     def tile(cls, op):
         inputs = op.inputs
-        check_chunks_unknown_shape(inputs, TilesError)
+        if has_unknown_shape(inputs):
+            yield
         axis_to_nsplits = defaultdict(list)
         has_dataframe = any(output_type == OutputType.dataframe
                             for output_type in op.output_types)
@@ -185,7 +183,11 @@ class LearnShuffle(LearnMapReduceOperand, LearnOperandMixin):
                 if ax < inp.ndim:
                     axis_to_nsplits[ax].append(inp.nsplits[ax])
         ax_nsplit = {ax: decide_unify_split(*ns) for ax, ns in axis_to_nsplits.items()}
-        inputs = [cls._safe_rechunk(inp, ax_nsplit) for inp in inputs]
+        rechunked_inputs = []
+        for inp in inputs:
+            inp = yield from cls._safe_rechunk(inp, ax_nsplit)
+            rechunked_inputs.append(inp)
+        inputs = rechunked_inputs
 
         mapper_seeds = [None] * len(op.axes)
         reducer_seeds = [None] * len(op.axes)
@@ -262,14 +264,12 @@ class LearnShuffle(LearnMapReduceOperand, LearnOperandMixin):
                 reduce_axes = tuple(ax for j, ax in enumerate(inp_axes) if reduce_sizes[j] > 1)
                 reduce_sizes_ = tuple(rs for rs in reduce_sizes if rs > 1)
                 for c in map_chunks:
-                    shuffle_key = ','.join(str(idx) for idx in c.index)
                     chunk_op = LearnShuffle(
                         stage=OperandStage.reduce,
                         output_types=output_types, axes=reduce_axes,
                         seeds=tuple(reducer_seeds[j][c.index[ax]] for j, ax in enumerate(inp_axes)
                                     if reduce_sizes[j] > 1),
-                        reduce_sizes=reduce_sizes_,
-                        shuffle_key=shuffle_key)
+                        reduce_sizes=reduce_sizes_)
                     params = cls._calc_chunk_params(c, inp_axes, inp.chunk_shape, oup,
                                                     output_type, chunk_op, False)
                     reduce_chunk = chunk_op.new_chunk([proxy_chunk], kws=[params])
@@ -341,20 +341,16 @@ class LearnShuffle(LearnMapReduceOperand, LearnOperandMixin):
             index = list(out.index)
             for ax, ind in zip(axes, reduce_index):
                 index[ax] = ind
-            group_key = ','.join(str(i) for i in index)
             selected = x
             for ax, to_hash_ind in zip(axes, to_hash_inds):
                 slc = (slice(None),) * ax + (to_hash_ind == index[ax],)
                 selected = _safe_slice(selected, slc, op.output_types[0])
-            ctx[(out.key, group_key)] = selected
+            ctx[out.key, tuple(index)] = selected
 
     @classmethod
-    def execute_reduce(cls, ctx, op):
-        in_chunk = op.input
-        input_keys, input_indexes = get_shuffle_input_keys_idxes(in_chunk)
-        inputs = [ctx[(input_key, op.shuffle_key)] for input_key in input_keys]
+    def execute_reduce(cls, ctx, op: "LearnShuffle"):
         inputs_grid = np.empty(op.reduce_sizes, dtype=object)
-        for input_index, inp in zip(input_indexes, inputs):
+        for input_index, inp in op.iter_mapper_data_with_index(ctx):
             reduce_index = tuple(input_index[ax] for ax in op.axes)
             inputs_grid[reduce_index] = inp
         ret = cls._concat_grid(inputs_grid, op.axes, op.output_types[0])
