@@ -539,12 +539,14 @@ REMOTE_FUNCATION = 5001
 RUN_SCRIPT = 5002
 
 # vineyard
-TENSOR_FROM_VINEYARD = 4002
-TENSOR_STORE_VINEYARD_CHUNK = 4003
-TENSOR_STORE_VINEYARD_GLOBAL_META = 4005
-DATAFRAME_FROM_VINEYARD = 4007
-DATAFRAME_STORE_VINEYARD_CHUNK = 4008
-DATAFRAME_STORE_VINEYARD_GLOBAL_META = 4010
+TENSOR_FROM_VINEYARD_CHUNK = 4000
+TENSOR_FROM_VINEYARD_META = 4001
+TENSOR_STORE_VINEYARD_CHUNK = 4002
+TENSOR_STORE_VINEYARD_META = 4003
+DATAFRAME_FROM_VINEYARD_CHUNK = 4004
+DATAFRAME_FROM_VINEYARD_META = 4005
+DATAFRAME_STORE_VINEYARD_CHUNK = 4006
+DATAFRAME_STORE_VINEYARD_META = 4007
 
 CHOLESKY_FUSE = 999988
 
