@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,43 +14,47 @@
 # limitations under the License.
 
 import logging
+import math
 import os
 import subprocess  # nosec
 import sys
 import time
 from collections import namedtuple
+from typing import List, Optional
 
 import psutil
 
 from .lib import nvutils
+from .utils import get_bool_environ
 
 logger = logging.getLogger(__name__)
 
-CGROUP_CPU_STAT_FILE = '/sys/fs/cgroup/cpuacct/cpuacct.stat'
+CGROUP_CPU_STAT_FILE = '/sys/fs/cgroup/cpuacct/cpuacct.usage'
 CGROUP_MEM_STAT_FILE = '/sys/fs/cgroup/memory/memory.stat'
 
 _proc = psutil.Process()
 _timer = getattr(time, 'monotonic', time.time)
 
-_cpu_use_process_stat = bool(int(os.environ.get('MARS_CPU_USE_PROCESS_STAT', '0').strip('"')))
-_cpu_use_cgroup_stat = bool(int(os.environ.get('MARS_CPU_USE_CGROUP_STAT', '0').strip('"')))
-_mem_use_process_stat = bool(int(os.environ.get('MARS_MEM_USE_PROCESS_STAT', '0').strip('"')))
-_mem_use_cgroup_stat = bool(int(os.environ.get('MARS_MEM_USE_CGROUP_STAT', '0').strip('"')))
+_use_process_stat = get_bool_environ('MARS_USE_PROCESS_STAT')
+_use_cgroup_stat = get_bool_environ('MARS_USE_CGROUP_STAT')
+_cpu_use_process_stat = get_bool_environ('MARS_CPU_USE_PROCESS_STAT')
+_cpu_use_cgroup_stat = get_bool_environ('MARS_CPU_USE_CGROUP_STAT')
+_mem_use_process_stat = get_bool_environ('MARS_MEM_USE_PROCESS_STAT')
+_mem_use_cgroup_stat = get_bool_environ('MARS_MEM_USE_CGROUP_STAT')
 
-if 'MARS_USE_PROCESS_STAT' in os.environ:
-    _cpu_use_process_stat = _mem_use_process_stat = \
-        bool(int(os.environ['MARS_USE_PROCESS_STAT'].strip('"')))
-if 'MARS_USE_CGROUP_STAT' in os.environ:
-    _cpu_use_cgroup_stat = _mem_use_cgroup_stat = \
-        bool(int(os.environ['MARS_USE_CGROUP_STAT'].strip('"')))
+# if general config exists, overwrite individual ones
+if _use_process_stat is not None:
+    _cpu_use_process_stat = _mem_use_process_stat = _use_process_stat
+if _use_cgroup_stat is not None:
+    _cpu_use_cgroup_stat = _mem_use_cgroup_stat = _use_cgroup_stat
 
 if 'MARS_CPU_TOTAL' in os.environ:
-    _cpu_total = int(os.environ['MARS_CPU_TOTAL'].strip('"'))
+    _cpu_total = int(math.ceil(float(os.environ['MARS_CPU_TOTAL'])))
 else:
     _cpu_total = psutil.cpu_count(logical=True)
 
 if 'MARS_MEMORY_TOTAL' in os.environ:
-    _mem_total = int(os.environ['MARS_MEMORY_TOTAL'].strip('"'))
+    _mem_total = int(os.environ['MARS_MEMORY_TOTAL'])
 else:
     _mem_total = None
 
@@ -78,7 +82,7 @@ def _read_cgroup_stat_file():
 _root_pid = None
 
 
-def virtual_memory():
+def virtual_memory() -> _virt_memory_stat:
     global _root_pid
 
     sys_mem = psutil.virtual_memory()
@@ -219,12 +223,35 @@ def iowait():
         return None
 
 
-_last_disk_io_meta = None
+_last_disk_io_metas = dict()
+_path_to_device = dict()
 _win_diskperf_called = False
 
 
-def disk_io_usage():
-    global _last_disk_io_meta, _win_diskperf_called
+def get_path_device(path: str):
+    for part in psutil.disk_partitions(all=True):
+        if path.startswith(part.mountpoint):
+            return part.device
+    return None
+
+
+def _get_path_device(path: str):
+    if path in _path_to_device:
+        return _path_to_device[path]
+
+    for part in psutil.disk_partitions(all=True):
+        if path.startswith(part.mountpoint):
+            dev_name = _path_to_device[path] = part.device.replace('/dev/', '')
+            return dev_name
+    _path_to_device[path] = None
+    return None
+
+
+_disk_io_usage_type = namedtuple('_disk_io_usage_type', 'reads writes')
+
+
+def disk_io_usage(path=None) -> Optional[_disk_io_usage_type]:
+    global _win_diskperf_called
 
     # Needed by psutil.disk_io_counters() under newer version of Windows.
     # diskperf -y need to be called or no disk information can be found.
@@ -238,22 +265,31 @@ def disk_io_usage():
             pass
         _win_diskperf_called = True
 
-    disk_counters = psutil.disk_io_counters()
+    if path is None:
+        disk_counters = psutil.disk_io_counters()
+    else:
+        dev_to_counters = psutil.disk_io_counters(perdisk=True)
+        disk_counters = dev_to_counters.get(_get_path_device(path))
+        if disk_counters is None:
+            return None
     tst = time.time()
 
     read_bytes = disk_counters.read_bytes
     write_bytes = disk_counters.write_bytes
-    if _last_disk_io_meta is None:
-        _last_disk_io_meta = (read_bytes, write_bytes, tst)
+    if path not in _last_disk_io_metas:
+        _last_disk_io_metas[path] = (read_bytes, write_bytes, tst)
         return None
 
-    last_read_bytes, last_write_bytes, last_time = _last_disk_io_meta
+    last_read_bytes, last_write_bytes, last_time = _last_disk_io_metas[path]
     delta_time = tst - last_time
+    if delta_time == 0:
+        return None
+
     read_speed = (read_bytes - last_read_bytes) / delta_time
     write_speed = (write_bytes - last_write_bytes) / delta_time
 
-    _last_disk_io_meta = (read_bytes, write_bytes, tst)
-    return read_speed, write_speed
+    _last_disk_io_metas[path] = (read_bytes, write_bytes, tst)
+    return _disk_io_usage_type(read_speed, write_speed)
 
 
 _last_net_io_meta = None
@@ -273,6 +309,9 @@ def net_io_usage():
 
     last_send_bytes, last_recv_bytes, last_time = _last_net_io_meta
     delta_time = tst - last_time
+    if delta_time == 0:
+        return None
+
     recv_speed = (recv_bytes - last_recv_bytes) / delta_time
     send_speed = (send_bytes - last_send_bytes) / delta_time
 
@@ -301,7 +340,7 @@ def cuda_count():
     return nvutils.get_device_count() or 0
 
 
-def cuda_card_stats():  # pragma: no cover
+def cuda_card_stats() -> List[_cuda_card_stat]:  # pragma: no cover
     infos = []
     device_count = nvutils.get_device_count()
     if not device_count:
