@@ -19,9 +19,10 @@ from .datasource import tensor, array, asarray, ascontiguousarray, asfortranarra
     scalar, empty, empty_like, ones, ones_like, zeros, zeros_like, \
     full, full_like, arange, diag, diagflat, eye, identity, linspace, \
     meshgrid, indices, tril, triu, fromtiledb, fromtiledb as from_tiledb, from_dataframe, \
-    fromhdf5, fromhdf5 as from_hdf5, fromzarr, fromzarr as from_zarr
+    fromhdf5, fromhdf5 as from_hdf5, fromzarr, fromzarr as from_zarr, \
+    fromvineyard, fromvineyard as from_vineyard
 from .datastore import totiledb, totiledb as to_tiledb, tohdf5, tohdf5 as to_hdf5, \
-    tozarr, tozarr as to_zarr  # pylint: disable=reimported
+    tozarr, tozarr as to_zarr, tovineyard, tovineyard as to_vineyard  # pylint: disable=reimported
 from .base import result_type, ndim, copyto, transpose, where, broadcast_to, broadcast_arrays, \
     expand_dims, rollaxis, swapaxes, moveaxis, ravel, atleast_1d, atleast_2d, atleast_3d, argwhere, \
     array_split, split, hsplit, vsplit, dsplit, roll, squeeze, diff, ediff1d, \
