@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,12 +19,13 @@ import itertools
 import numpy as np
 
 from ...core import ExecutableTuple
-from ...serialize import ValueType, AnyField, DictField, KeyField, StringField, ListField
-from ...utils import check_chunks_unknown_shape
-from ...tiles import TilesError
+from ...serialization.serializables import FieldTypes, AnyField, DictField, \
+    KeyField, StringField, ListField
+from ...utils import has_unknown_shape
 from ..core import Tensor, TensorOrder
 from ..datasource import tensor as astensor
-from ..utils import unify_chunks, broadcast_shape, check_out_param, filter_inputs, check_order
+from ..utils import unify_chunks, broadcast_shape, check_out_param, \
+    filter_inputs, check_order
 from ..operands import TensorOperandMixin, TensorOperand
 from ..array_utils import device, as_same_device, convert_order
 
@@ -35,8 +36,10 @@ class TensorElementWise(TensorOperandMixin):
     @classmethod
     def tile(cls, op):
         if len(op.inputs) > 1:
-            check_chunks_unknown_shape(op.inputs, TilesError)
-        inputs = unify_chunks(*[(input, list(range(input.ndim))[::-1]) for input in op.inputs])
+            if has_unknown_shape(*op.inputs):
+                yield
+        inputs = yield from unify_chunks(
+            *[(input, list(range(input.ndim))[::-1]) for input in op.inputs])
 
         chunk_shapes = [t.chunk_shape for t in inputs]
         out_chunk_shape = broadcast_shape(*chunk_shapes)
@@ -58,8 +61,12 @@ class TensorElementWise(TensorOperandMixin):
                 nsplits[i][idx] = s
 
         new_op = op.copy().reset_key()
-        kws = [{'chunks': out_chunk, 'nsplits': nsplits, 'shape': o.shape, 'dtype': o.dtype}
-               for out_chunk, o in zip(out_chunks, op.outputs)]
+        kws = []
+        for out_chunk, o in zip(out_chunks, op.outputs):
+            params = o.params.copy()
+            params['chunks'] = out_chunk
+            params['nsplits'] = nsplits
+            kws.append(params)
         return new_op.new_tensors(list(inputs), kws=kws, output_limit=len(op.outputs))
 
 
@@ -142,7 +149,7 @@ class TensorBinOp(TensorOperand, TensorBinOpMixin):
     _where = KeyField('where')
     _casting = StringField('casting')
     _order = StringField('order')
-    _err = DictField('err', ValueType.string, ValueType.string)
+    _err = DictField('err', FieldTypes.string, FieldTypes.string)
 
     def __init__(self, lhs=None, rhs=None, out=None, where=None, order=None, **kwargs):
         super().__init__(_lhs=lhs, _rhs=rhs, _out=out, _where=where, _order=order, **kwargs)
@@ -186,7 +193,7 @@ class TensorBinOp(TensorOperand, TensorBinOpMixin):
         inputs_iter = iter(inputs)
         x1 = self._lhs if np.isscalar(self._lhs) else next(inputs_iter)
         x2 = self._rhs if np.isscalar(self._rhs) else next(inputs_iter)
-        setattr(self, '_sparse', self._is_sparse(x1, x2))
+        setattr(self, 'sparse', self._is_sparse(x1, x2))
 
     def _set_inputs(self, inputs):
         super()._set_inputs(inputs)
@@ -281,7 +288,7 @@ class TensorBinOp(TensorOperand, TensorBinOpMixin):
         # if `out` is specified, use out's dtype and shape
         if t.shape != out_shape:
             t = self.new_tensor(inputs, out_shape, order=order)
-        setattr(self, '_dtype', out_dtype)
+        setattr(self, 'dtype', out_dtype)
 
         out.data = t.data
         return out
@@ -346,7 +353,7 @@ class TensorUnaryOp(TensorOperand, TensorUnaryOpMixin):
     _where = KeyField('where')
     _casting = StringField('casting')
     _order = StringField('order')
-    _err = DictField('err', ValueType.string, ValueType.string)
+    _err = DictField('err', FieldTypes.string, FieldTypes.string)
 
     def __init__(self, out=None, where=None, order=None, **kwargs):
         super().__init__(_out=out, _where=where, _order=order, **kwargs)
@@ -412,7 +419,7 @@ class TensorUnaryOp(TensorOperand, TensorUnaryOpMixin):
         return x, out, where
 
     def _set_sparse(self, inputs):
-        setattr(self, '_sparse', self._is_sparse(inputs[0]))
+        setattr(self, 'sparse', self._is_sparse(inputs[0]))
 
     def _calc_order(self, x, out):
         if out is not None:
@@ -461,7 +468,7 @@ class TensorUnaryOp(TensorOperand, TensorUnaryOpMixin):
         # if `out` is specified, use out's dtype and shape
         if t.shape != out_shape:
             t = self.new_tensor(inputs, out_shape, order=order)
-        setattr(self, '_dtype', out_dtype)
+        setattr(self, 'dtype', out_dtype)
 
         out.data = t.data
         return out
@@ -550,7 +557,7 @@ class TensorOutBinOp(TensorOperand, TensorElementWiseWithInputs):
         return False
 
     def _set_sparse(self, inputs):
-        setattr(self, '_sparse', self._is_sparse(inputs[0]))
+        setattr(self, 'sparse', self._is_sparse(inputs[0]))
 
     @property
     def _fun(self):
@@ -622,12 +629,12 @@ class TensorMultiOp(TensorElementWiseWithInputs, TensorOperand):
     _where = KeyField('where')
     _casting = StringField('casting')
     _order = StringField('order')
-    _err = DictField('err', ValueType.string, ValueType.string)
+    _err = DictField('err', FieldTypes.string, FieldTypes.string)
 
-    def __init__(self, args=None, out=None, where=None, dtype=None, casting=None,
+    def __init__(self, args=None, out=None, where=None, casting=None,
                  order=None, err=None, **kwargs):
         super().__init__(_args=args, _out=out, _where=where, _order=order,
-                         _dtype=dtype, _casting=casting, _er=err, **kwargs)
+                         _casting=casting, _er=err, **kwargs)
         if self._casting is None:
             self._casting = 'same_kind'
         if self._order is None:
@@ -664,7 +671,7 @@ class TensorMultiOp(TensorElementWiseWithInputs, TensorOperand):
         for idx in range(len(self._args)):
             if not np.isscalar(self._args[idx]):
                 args[idx] = next(inputs_iter)
-        setattr(self, '_sparse', self._is_sparse(*args))
+        setattr(self, 'sparse', self._is_sparse(*args))
 
     def _set_inputs(self, inputs):
         super()._set_inputs(inputs)
@@ -713,7 +720,7 @@ class TensorMultiOp(TensorElementWiseWithInputs, TensorOperand):
         # if `out` is specified, use out's dtype and shape
         if t.shape != out_shape:
             t = self.new_tensor(inputs, out_shape, order=order)
-        setattr(self, '_dtype', out_dtype)
+        setattr(self, 'dtype', out_dtype)
 
         out.data = t.data
         return out
