@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@
 import numpy as np
 
 from ... import opcodes as OperandDef
-from ...serialize import ValueType, KeyField, ListField
+from ...serialization.serializables import FieldTypes, KeyField, ListField
 from ..operands import TensorHasInput, TensorOperandMixin
 from ..datasource import tensor as astensor
 from ..array_utils import as_same_device, device
@@ -35,12 +35,12 @@ class TensorTranspose(TensorHasInput, TensorOperandMixin):
     _op_type_ = OperandDef.TRANSPOSE
 
     _input = KeyField('input')
-    _axes = ListField('axes', ValueType.int32)
+    _axes = ListField('axes', FieldTypes.int32)
 
-    def __init__(self, axes=None, dtype=None, sparse=False, **kw):
-        super().__init__(_axes=axes, _dtype=dtype, _sparse=sparse,
+    def __init__(self, axes=None, **kw):
+        super().__init__(_axes=axes,
                          # transpose will create a view
-                         _create_view=True, **kw)
+                         create_view=True, **kw)
 
     @property
     def axes(self):
