@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,7 +13,7 @@
 # limitations under the License.
 
 
-from .array import tensor, named_tensor, array, asarray, ascontiguousarray, asfortranarray, \
+from .array import tensor, array, asarray, ascontiguousarray, asfortranarray, \
     ArrayDataSource, CSRMatrixDataSource
 from .scalar import scalar, Scalar
 from .empty import empty, empty_like, TensorEmpty, TensorEmptyLike
@@ -35,4 +35,4 @@ from .from_tiledb import fromtiledb, TensorTileDBDataSource
 from .from_hdf5 import fromhdf5, TensorHDF5DataSource
 from .from_zarr import fromzarr, TensorFromZarr
 from .from_dataframe import from_dataframe, from_series, TensorFromDataFrame
-from .from_vineyard import from_vineyard, TensorFromVineyard, TensorFromVineyardChunk
+from .from_vineyard import fromvineyard, TensorFromVineyard, TensorFromVineyardChunk
