@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,6 +16,5 @@
 
 from .to_tiledb import totiledb, TensorTileDBDataStore, TensorTileDBConsolidate
 from .to_hdf5 import tohdf5, TensorHDF5DataStore
+from .to_vineyard import tovineyard, TensorVineyardDataStoreChunk, TensorVineyardDataStoreMeta
 from .to_zarr import tozarr, TensorToZarrDataStore
-from .to_vineyard import tovineyard, TensorVineyardDataStoreGlobalMeta, \
-    TensorVineyardDataStoreChunk, TensorVineyardDataStoreChunkMap
