@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,11 +17,11 @@ from typing import List, Tuple
 import numpy as np
 
 from .... import opcodes as OperandDef
-from ....operands import OperandStage
-from ....serialize import ValueType, KeyField, AnyField, Float16Field, Int32Field, TupleField
-from ....tiles import TilesError
-from ....utils import check_chunks_unknown_shape, get_shuffle_input_keys_idxes, \
-    require_module
+from ....core import recursive_tile
+from ....core.operand import OperandStage
+from ....serialization.serializables import FieldTypes, KeyField, \
+    AnyField, Float16Field, Int32Field, TupleField
+from ....utils import has_unknown_shape, require_module, ensure_own_data
 from ....config import options
 from ...operands import TensorMapReduceOperand, TensorOperandMixin, TensorShuffleProxy
 from ...array_utils import as_same_device, device, cp
@@ -44,16 +44,15 @@ class TensorPdist(TensorMapReduceOperand, TensorOperandMixin):
     _a_offset = Int32Field('a_offset')
     _b = KeyField('b')
     _b_offset = Int32Field('b_offset')
-    _out_sizes = TupleField('out_sizes', ValueType.int32)
+    _out_sizes = TupleField('out_sizes', FieldTypes.int32)
     _n = Int32Field('n')
 
     def __init__(self, metric=None, p=None, w=None, v=None, vi=None,
                  a=None, a_offset=None, b=None, b_offset=None, out_sizes=None, n=None,
-                 aggregate_size=None, stage=None, shuffle_key=None, dtype=None, **kw):
+                 aggregate_size=None, **kw):
         super().__init__(_metric=metric, _p=p, _w=w, _v=v, _vi=vi,
                          _a=a, _a_offset=a_offset, _b=b, _b_offset=b_offset, _out_sizes=out_sizes,
-                         _n=n, _dtype=dtype, _aggregate_size=aggregate_size, _stage=stage,
-                         _shuffle_key=shuffle_key, **kw)
+                         _n=n, _aggregate_size=aggregate_size, **kw)
 
     def _set_inputs(self, inputs: List) -> None:
         super()._set_inputs(inputs)
@@ -214,7 +213,7 @@ class TensorPdist(TensorMapReduceOperand, TensorOperandMixin):
         reduce_chunks = []
         for p in range(aggregate_size):
             reduce_chunk_op = TensorPdist(
-                stage=OperandStage.reduce, shuffle_key=str(p), dtype=out_tensor.dtype)
+                stage=OperandStage.reduce, dtype=out_tensor.dtype)
             reduce_chunk = reduce_chunk_op.new_chunk(
                 [proxy_chunk], shape=(out_sizes[p],), order=out_tensor.order,
                 index=(p,))
@@ -229,15 +228,17 @@ class TensorPdist(TensorMapReduceOperand, TensorOperandMixin):
     @classmethod
     def tile(cls, op):
         # make sure every inputs have known shape
-        check_chunks_unknown_shape(op.inputs, TilesError)
+        if has_unknown_shape(*op.inputs):
+            yield
 
-        in_tensor = op.input.rechunk({1: op.input.shape[1]})._inplace_tile()
+        in_tensor = yield from recursive_tile(
+            op.input.rechunk({1: op.input.shape[1]}))
         # rechunk w, v, vi into one chunk if any of them has value
         extra_inputs = [None] * 3
         for i, ei in enumerate([op.w, op.v, op.vi]):
             if ei is None:
                 continue
-            new_ei = ei.rechunk(ei.shape)._inplace_tile()
+            new_ei = yield from recursive_tile(ei.rechunk(ei.shape))
             extra_inputs[i] = new_ei
         w, v, vi = extra_inputs
 
@@ -277,13 +278,14 @@ class TensorPdist(TensorMapReduceOperand, TensorOperandMixin):
 
             if b is None:
                 # one input, pdist on same chunk
-                dists = pdist(a, metric=metric, **kw)
+                dists = pdist(ensure_own_data(a), metric=metric, **kw)
                 i_indices, j_indices = xp.triu_indices(a.shape[0], k=1)
                 i_indices += op.a_offset
                 j_indices += op.a_offset
             else:
                 # two inputs, pdist on different chunks
-                dists = cdist(a, b, metric=metric, **kw).ravel()
+                dists = cdist(ensure_own_data(a), ensure_own_data(b),
+                              metric=metric, **kw).ravel()
                 mgrid = \
                     xp.mgrid[op.a_offset: op.a_offset + a.shape[0],
                     op.b_offset: op.b_offset + b.shape[0]]
@@ -294,7 +296,7 @@ class TensorPdist(TensorMapReduceOperand, TensorOperandMixin):
             out_row_cum_sizes[0] = 0
             xp.cumsum(out_row_sizes, out=out_row_cum_sizes[1:])
             indices = out_row_cum_sizes[i_indices] + j_indices - \
-                      (op.n - out_row_sizes[i_indices])
+                (op.n - out_row_sizes[i_indices])
 
             # save as much memory as possible
             del i_indices, j_indices, out_row_sizes, out_row_cum_sizes
@@ -307,7 +309,7 @@ class TensorPdist(TensorMapReduceOperand, TensorOperandMixin):
                 to_filter = (indices >= start_index) & (indices < end_index)
                 downside_indices = indices[to_filter] - start_index
                 downside_dists = dists[to_filter]
-                ctx[out.key, str(i)] = (downside_indices, downside_dists)
+                ctx[out.key, (i,)] = (downside_indices, downside_dists)
 
     @classmethod
     def _execute_single(cls, ctx, op):
@@ -332,12 +334,12 @@ class TensorPdist(TensorMapReduceOperand, TensorOperandMixin):
             if op.vi is not None:
                 kw['VI'] = next(inputs_iter)
 
-        ctx[op.outputs[0].key] = pdist(x, metric=op.metric, **kw)
+        ctx[op.outputs[0].key] = pdist(ensure_own_data(x),
+                                       metric=op.metric, **kw)
 
     @classmethod
-    def _execute_reduce(cls, ctx, op):
-        input_keys, _ = get_shuffle_input_keys_idxes(op.inputs[0])
-        raw_inputs = [ctx[(input_key, op.shuffle_key)] for input_key in input_keys]
+    def _execute_reduce(cls, ctx, op: "TensorPdist"):
+        raw_inputs = list(op.iter_mapper_data(ctx))
         raw_indices = [inp[0] for inp in raw_inputs]
         raw_dists = [inp[1] for inp in raw_inputs]
         inputs, device_id, xp = as_same_device(
